@@ -115,19 +115,6 @@
 
 G_BEGIN_DECLS
 
-/* These are used to get/set the working days in the week. The bit-flags are
- * combined together. The bits must be from 0 (Sun) to 6 (Sat) to match the
- * day values used by localtime etc. */
-typedef enum {
-	E_DAY_VIEW_SUNDAY	= 1 << 0,
-	E_DAY_VIEW_MONDAY	= 1 << 1,
-	E_DAY_VIEW_TUESDAY	= 1 << 2,
-	E_DAY_VIEW_WEDNESDAY	= 1 << 3,
-	E_DAY_VIEW_THURSDAY	= 1 << 4,
-	E_DAY_VIEW_FRIDAY	= 1 << 5,
-	E_DAY_VIEW_SATURDAY	= 1 << 6
-} EDayViewDays;
-
 /* These are used to specify the type of an appointment. They match those
  * used in EMeetingTimeSelector. */
 typedef enum {
@@ -236,14 +223,6 @@ struct _EDayView {
 	time_t lower;
 	time_t upper;
 
-	/* Whether we are showing the work-week view. */
-	gboolean work_week_view;
-
-	/* The number of days we are showing. Usually 1 or 5, but can be up
-	 * to E_DAY_VIEW_MAX_DAYS, e.g. when the user selects a range of
-	 * days in the date navigator. */
-	gint days_shown;
-
 	/* The start of each day & an extra element to hold the last time. */
 	time_t day_starts[E_DAY_VIEW_MAX_DAYS + 1];
 
@@ -289,14 +268,6 @@ struct _EDayView {
 	gint last_hour_shown;
 	gint last_minute_shown;
 
-	/* Bitwise combination of working days. Defaults to Mon-Fri. */
-	EDayViewDays working_days;
-
-	/* Whether we show the Marcus Bains Line in the main canvas and time canvas. */
-	gboolean marcus_bains_show_line;
-	gchar *marcus_bains_day_view_color;
-	gchar *marcus_bains_time_bar_color;
-
 	/* Whether we use show event end times in the main canvas. */
 	gboolean show_event_end_times;
 
@@ -493,12 +464,6 @@ gint		e_day_view_get_days_shown	(EDayView *day_view);
 void		e_day_view_set_days_shown	(EDayView *day_view,
 						 gint days_shown);
 
-/* This specifies the working days in the week. The value is a bitwise
- * combination of day flags. Defaults to Mon-Fri. */
-EDayViewDays	e_day_view_get_working_days	(EDayView *day_view);
-void		e_day_view_set_working_days	(EDayView *day_view,
-						 EDayViewDays days);
-
 /* Whether we display the Marcus Bains Line in the main canvas and time
  * canvas. */
 void		e_day_view_marcus_bains_update	(EDayView *day_view);
@@ -599,8 +564,6 @@ void		e_day_view_ensure_rows_visible	(EDayView *day_view,
 						 gint start_row,
 						 gint end_row);
 
-gboolean	e_day_view_is_editing		(EDayView *day_view);
-
 G_END_DECLS
 
 #endif /* E_DAY_VIEW_H */
