@@ -57,6 +57,8 @@
 #define ATTACHMENT_QUERY "standard::*,preview::*,thumbnail::*"
 
 struct _EAttachmentPrivate {
+	GMutex property_lock;
+
 	GFile *file;
 	GIcon *icon;
 	GFileInfo *file_info;
@@ -80,6 +82,13 @@ struct _EAttachmentPrivate {
 	 * If we are removed from the store, we lazily free the
 	 * reference when it is found to be to be invalid. */
 	GtkTreeRowReference *reference;
+
+	/* These are IDs for idle callbacks,
+	 * protected by the idle_lock mutex. */
+	GMutex idle_lock;
+	guint update_icon_column_idle_id;
+	guint update_progress_columns_idle_id;
+	guint update_file_info_columns_idle_id;
 };
 
 enum {
@@ -110,48 +119,56 @@ create_system_thumbnail (EAttachment *attachment,
 {
 	GFile *file;
 	GFile *icon_file;
+	gchar *file_path = NULL;
 	gchar *thumbnail = NULL;
+	gboolean success = FALSE;
 
-	g_return_val_if_fail (attachment != NULL, FALSE);
+	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), FALSE);
 	g_return_val_if_fail (icon != NULL, FALSE);
 
-	file = e_attachment_get_file (attachment);
+	file = e_attachment_ref_file (attachment);
+	if (file != NULL)
+		file_path = g_file_get_path (file);
 
-	if (file && g_file_has_uri_scheme (file, "file")) {
-		gchar *path = g_file_get_path (file);
-		if (path) {
-			thumbnail = e_icon_factory_create_thumbnail (path);
-			g_free (path);
-		}
+	if (file_path != NULL) {
+		thumbnail = e_icon_factory_create_thumbnail (file_path);
+		g_free (file_path);
 	}
 
 	if (thumbnail == NULL)
-		return FALSE;
+		goto exit;
 
 	icon_file = g_file_new_for_path (thumbnail);
 
-	if (*icon)
+	if (*icon != NULL)
 		g_object_unref (*icon);
 
 	*icon = g_file_icon_new (icon_file);
 
 	g_object_unref (icon_file);
 
-	if (file) {
+	if (file != NULL) {
 		GFileInfo *file_info;
 		const gchar *attribute;
 
-		file_info = e_attachment_get_file_info (attachment);
+		file_info = e_attachment_ref_file_info (attachment);
 		attribute = G_FILE_ATTRIBUTE_THUMBNAIL_PATH;
 
-		if (file_info != NULL)
+		if (file_info != NULL) {
 			g_file_info_set_attribute_byte_string (
 				file_info, attribute, thumbnail);
+			g_object_unref (file_info);
+		}
 	}
 
 	g_free (thumbnail);
 
-	return TRUE;
+	success = TRUE;
+
+exit:
+	g_clear_object (&file);
+
+	return success;
 }
 
 static gchar *
@@ -185,29 +202,38 @@ attachment_get_default_charset (void)
 	return charset;
 }
 
-static void
-attachment_update_file_info_columns (EAttachment *attachment)
+static gboolean
+attachment_update_file_info_columns_idle_cb (gpointer weak_ref)
 {
+	EAttachment *attachment;
 	GtkTreeRowReference *reference;
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	GFileInfo *file_info;
 	const gchar *content_type;
-	const gchar *description;
 	const gchar *display_name;
 	gchar *content_desc;
 	gchar *display_size;
+	gchar *description;
 	gchar *caption;
 	goffset size;
 
+	attachment = g_weak_ref_get (weak_ref);
+	if (attachment == NULL)
+		goto exit;
+
+	g_mutex_lock (&attachment->priv->idle_lock);
+	attachment->priv->update_file_info_columns_idle_id = 0;
+	g_mutex_unlock (&attachment->priv->idle_lock);
+
 	reference = e_attachment_get_reference (attachment);
 	if (!gtk_tree_row_reference_valid (reference))
-		return;
+		goto exit;
 
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 	if (file_info == NULL)
-		return;
+		goto exit;
 
 	model = gtk_tree_row_reference_get_model (reference);
 	path = gtk_tree_row_reference_get_path (reference);
@@ -221,9 +247,11 @@ attachment_update_file_info_columns (EAttachment *attachment)
 	content_desc = g_content_type_get_description (content_type);
 	display_size = g_format_size (size);
 
-	description = e_attachment_get_description (attachment);
-	if (description == NULL || *description == '\0')
-		description = display_name;
+	description = e_attachment_dup_description (attachment);
+	if (description == NULL || *description == '\0') {
+		g_free (description);
+		description = g_strdup (display_name);
+	}
 
 	if (size > 0)
 		caption = g_strdup_printf (
@@ -241,12 +269,40 @@ attachment_update_file_info_columns (EAttachment *attachment)
 
 	g_free (content_desc);
 	g_free (display_size);
+	g_free (description);
 	g_free (caption);
+
+	g_clear_object (&file_info);
+
+exit:
+	g_clear_object (&attachment);
+
+	return FALSE;
 }
 
 static void
-attachment_update_icon_column (EAttachment *attachment)
+attachment_update_file_info_columns (EAttachment *attachment)
 {
+	g_mutex_lock (&attachment->priv->idle_lock);
+
+	if (attachment->priv->update_file_info_columns_idle_id == 0) {
+		guint idle_id;
+
+		idle_id = g_idle_add_full (
+			G_PRIORITY_HIGH_IDLE,
+			attachment_update_file_info_columns_idle_cb,
+			e_weak_ref_new (attachment),
+			(GDestroyNotify) e_weak_ref_free);
+		attachment->priv->update_file_info_columns_idle_id = idle_id;
+	}
+
+	g_mutex_unlock (&attachment->priv->idle_lock);
+}
+
+static gboolean
+attachment_update_icon_column_idle_cb (gpointer weak_ref)
+{
+	EAttachment *attachment;
 	GtkTreeRowReference *reference;
 	GtkTreeModel *model;
 	GtkTreePath *path;
@@ -257,9 +313,17 @@ attachment_update_icon_column (EAttachment *attachment)
 	const gchar *emblem_name = NULL;
 	const gchar *thumbnail_path = NULL;
 
+	attachment = g_weak_ref_get (weak_ref);
+	if (attachment == NULL)
+		goto exit;
+
+	g_mutex_lock (&attachment->priv->idle_lock);
+	attachment->priv->update_icon_column_idle_id = 0;
+	g_mutex_unlock (&attachment->priv->idle_lock);
+
 	reference = e_attachment_get_reference (attachment);
 	if (!gtk_tree_row_reference_valid (reference))
-		return;
+		goto exit;
 
 	model = gtk_tree_row_reference_get_model (reference);
 	path = gtk_tree_row_reference_get_path (reference);
@@ -267,7 +331,7 @@ attachment_update_icon_column (EAttachment *attachment)
 	gtk_tree_path_free (path);
 
 	cancellable = attachment->priv->cancellable;
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 
 	if (file_info != NULL) {
 		icon = g_file_info_get_icon (file_info);
@@ -370,11 +434,38 @@ attachment_update_icon_column (EAttachment *attachment)
 		g_object_unref (attachment->priv->icon);
 	attachment->priv->icon = icon;
 	g_object_notify (G_OBJECT (attachment), "icon");
+
+	g_clear_object (&file_info);
+
+exit:
+	g_clear_object (&attachment);
+
+	return FALSE;
 }
 
 static void
-attachment_update_progress_columns (EAttachment *attachment)
+attachment_update_icon_column (EAttachment *attachment)
 {
+	g_mutex_lock (&attachment->priv->idle_lock);
+
+	if (attachment->priv->update_icon_column_idle_id == 0) {
+		guint idle_id;
+
+		idle_id = g_idle_add_full (
+			G_PRIORITY_HIGH_IDLE,
+			attachment_update_icon_column_idle_cb,
+			e_weak_ref_new (attachment),
+			(GDestroyNotify) e_weak_ref_free);
+		attachment->priv->update_icon_column_idle_id = idle_id;
+	}
+
+	g_mutex_unlock (&attachment->priv->idle_lock);
+}
+
+static gboolean
+attachment_update_progress_columns_idle_cb (gpointer weak_ref)
+{
+	EAttachment *attachment;
 	GtkTreeRowReference *reference;
 	GtkTreeModel *model;
 	GtkTreePath *path;
@@ -383,9 +474,17 @@ attachment_update_progress_columns (EAttachment *attachment)
 	gboolean saving;
 	gint percent;
 
+	attachment = g_weak_ref_get (weak_ref);
+	if (attachment == NULL)
+		goto exit;
+
+	g_mutex_lock (&attachment->priv->idle_lock);
+	attachment->priv->update_progress_columns_idle_id = 0;
+	g_mutex_unlock (&attachment->priv->idle_lock);
+
 	reference = e_attachment_get_reference (attachment);
 	if (!gtk_tree_row_reference_valid (reference))
-		return;
+		goto exit;
 
 	model = gtk_tree_row_reference_get_model (reference);
 	path = gtk_tree_row_reference_get_path (reference);
@@ -403,6 +502,30 @@ attachment_update_progress_columns (EAttachment *attachment)
 		E_ATTACHMENT_STORE_COLUMN_PERCENT, percent,
 		E_ATTACHMENT_STORE_COLUMN_SAVING, saving,
 		-1);
+
+exit:
+	g_clear_object (&attachment);
+
+	return FALSE;
+}
+
+static void
+attachment_update_progress_columns (EAttachment *attachment)
+{
+	g_mutex_lock (&attachment->priv->idle_lock);
+
+	if (attachment->priv->update_progress_columns_idle_id == 0) {
+		guint idle_id;
+
+		idle_id = g_idle_add_full (
+			G_PRIORITY_HIGH_IDLE,
+			attachment_update_progress_columns_idle_cb,
+			e_weak_ref_new (attachment),
+			(GDestroyNotify) e_weak_ref_free);
+		attachment->priv->update_progress_columns_idle_id = idle_id;
+	}
+
+	g_mutex_unlock (&attachment->priv->idle_lock);
 }
 
 static void
@@ -535,7 +658,7 @@ attachment_set_property (GObject *object,
 		case PROP_MIME_PART:
 			e_attachment_set_mime_part (
 				E_ATTACHMENT (object),
-				g_value_get_boxed (value));
+				g_value_get_object (value));
 			return;
 
 		case PROP_REFERENCE:
@@ -563,79 +686,92 @@ attachment_get_property (GObject *object,
 	switch (property_id) {
 		case PROP_CAN_SHOW:
 			g_value_set_boolean (
-				value, e_attachment_get_can_show (
+				value,
+				e_attachment_get_can_show (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_DISPOSITION:
 			g_value_set_string (
-				value, e_attachment_get_disposition (
+				value,
+				e_attachment_get_disposition (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_ENCRYPTED:
 			g_value_set_int (
-				value, e_attachment_get_encrypted (
+				value,
+				e_attachment_get_encrypted (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_FILE:
-			g_value_set_object (
-				value, e_attachment_get_file (
+			g_value_take_object (
+				value,
+				e_attachment_ref_file (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_FILE_INFO:
-			g_value_set_object (
-				value, e_attachment_get_file_info (
+			g_value_take_object (
+				value,
+				e_attachment_ref_file_info (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_ICON:
-			g_value_set_object (
-				value, e_attachment_get_icon (
+			g_value_take_object (
+				value,
+				e_attachment_ref_icon (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_SHOWN:
 			g_value_set_boolean (
-				value, e_attachment_get_shown (
+				value,
+				e_attachment_get_shown (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_LOADING:
 			g_value_set_boolean (
-				value, e_attachment_get_loading (
+				value,
+				e_attachment_get_loading (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_MIME_PART:
-			g_value_set_boxed (
-				value, e_attachment_get_mime_part (
+			g_value_take_object (
+				value,
+				e_attachment_ref_mime_part (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_PERCENT:
 			g_value_set_int (
-				value, e_attachment_get_percent (
+				value,
+				e_attachment_get_percent (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_REFERENCE:
 			g_value_set_boxed (
-				value, e_attachment_get_reference (
+				value,
+				e_attachment_get_reference (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_SAVING:
 			g_value_set_boolean (
-				value, e_attachment_get_saving (
+				value,
+				e_attachment_get_saving (
 				E_ATTACHMENT (object)));
 			return;
 
 		case PROP_SIGNED:
 			g_value_set_int (
-				value, e_attachment_get_signed (
+				value,
+				e_attachment_get_signed (
 				E_ATTACHMENT (object)));
 			return;
 	}
@@ -650,30 +786,11 @@ attachment_dispose (GObject *object)
 
 	priv = E_ATTACHMENT_GET_PRIVATE (object);
 
-	if (priv->file != NULL) {
-		g_object_unref (priv->file);
-		priv->file = NULL;
-	}
-
-	if (priv->icon != NULL) {
-		g_object_unref (priv->icon);
-		priv->icon = NULL;
-	}
-
-	if (priv->file_info != NULL) {
-		g_object_unref (priv->file_info);
-		priv->file_info = NULL;
-	}
-
-	if (priv->cancellable != NULL) {
-		g_object_unref (priv->cancellable);
-		priv->cancellable = NULL;
-	}
-
-	if (priv->mime_part != NULL) {
-		g_object_unref (priv->mime_part);
-		priv->mime_part = NULL;
-	}
+	g_clear_object (&priv->file);
+	g_clear_object (&priv->icon);
+	g_clear_object (&priv->file_info);
+	g_clear_object (&priv->cancellable);
+	g_clear_object (&priv->mime_part);
 
 	if (priv->emblem_timeout_id > 0) {
 		g_source_remove (priv->emblem_timeout_id);
@@ -695,6 +812,18 @@ attachment_finalize (GObject *object)
 
 	priv = E_ATTACHMENT_GET_PRIVATE (object);
 
+	if (priv->update_icon_column_idle_id > 0)
+		g_source_remove (priv->update_icon_column_idle_id);
+
+	if (priv->update_progress_columns_idle_id > 0)
+		g_source_remove (priv->update_progress_columns_idle_id);
+
+	if (priv->update_file_info_columns_idle_id > 0)
+		g_source_remove (priv->update_file_info_columns_idle_id);
+
+	g_mutex_clear (&priv->property_lock);
+	g_mutex_clear (&priv->idle_lock);
+
 	g_free (priv->disposition);
 
 	/* Chain up to parent's finalize() method. */
@@ -867,6 +996,9 @@ e_attachment_init (EAttachment *attachment)
 	attachment->priv->encrypted = CAMEL_CIPHER_VALIDITY_ENCRYPT_NONE;
 	attachment->priv->signed_ = CAMEL_CIPHER_VALIDITY_SIGN_NONE;
 
+	g_mutex_init (&attachment->priv->property_lock);
+	g_mutex_init (&attachment->priv->idle_lock);
+
 	g_signal_connect (
 		attachment, "notify::encrypted",
 		G_CALLBACK (attachment_update_icon_column), NULL);
@@ -1007,7 +1139,7 @@ e_attachment_add_to_multipart (EAttachment *attachment,
 	g_return_if_fail (CAMEL_IS_MULTIPART (multipart));
 
 	/* Still loading?  Too bad. */
-	mime_part = e_attachment_get_mime_part (attachment);
+	mime_part = e_attachment_ref_mime_part (attachment);
 	if (mime_part == NULL)
 		return;
 
@@ -1078,6 +1210,8 @@ e_attachment_add_to_multipart (EAttachment *attachment,
 
 exit:
 	camel_multipart_add_part (multipart, mime_part);
+
+	g_clear_object (&mime_part);
 }
 
 void
@@ -1115,24 +1249,55 @@ e_attachment_get_disposition (EAttachment *attachment)
 	return attachment->priv->disposition;
 }
 
+gchar *
+e_attachment_dup_disposition (EAttachment *attachment)
+{
+	const gchar *protected;
+	gchar *duplicate;
+
+	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
+
+	g_mutex_lock (&attachment->priv->property_lock);
+
+	protected = e_attachment_get_disposition (attachment);
+	duplicate = g_strdup (protected);
+
+	g_mutex_unlock (&attachment->priv->property_lock);
+
+	return duplicate;
+}
+
 void
 e_attachment_set_disposition (EAttachment *attachment,
                               const gchar *disposition)
 {
 	g_return_if_fail (E_IS_ATTACHMENT (attachment));
 
+	g_mutex_lock (&attachment->priv->property_lock);
+
 	g_free (attachment->priv->disposition);
 	attachment->priv->disposition = g_strdup (disposition);
 
+	g_mutex_unlock (&attachment->priv->property_lock);
+
 	g_object_notify (G_OBJECT (attachment), "disposition");
 }
 
 GFile *
-e_attachment_get_file (EAttachment *attachment)
+e_attachment_ref_file (EAttachment *attachment)
 {
+	GFile *file = NULL;
+
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	return attachment->priv->file;
+	g_mutex_lock (&attachment->priv->property_lock);
+
+	if (attachment->priv->file != NULL)
+		file = g_object_ref (attachment->priv->file);
+
+	g_mutex_unlock (&attachment->priv->property_lock);
+
+	return file;
 }
 
 void
@@ -1146,20 +1311,31 @@ e_attachment_set_file (EAttachment *attachment,
 		g_object_ref (file);
 	}
 
-	if (attachment->priv->file != NULL)
-		g_object_unref (attachment->priv->file);
+	g_mutex_lock (&attachment->priv->property_lock);
 
+	g_clear_object (&attachment->priv->file);
 	attachment->priv->file = file;
 
+	g_mutex_unlock (&attachment->priv->property_lock);
+
 	g_object_notify (G_OBJECT (attachment), "file");
 }
 
 GFileInfo *
-e_attachment_get_file_info (EAttachment *attachment)
+e_attachment_ref_file_info (EAttachment *attachment)
 {
+	GFileInfo *file_info = NULL;
+
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	return attachment->priv->file_info;
+	g_mutex_lock (&attachment->priv->property_lock);
+
+	if (attachment->priv->file_info != NULL)
+		file_info = g_object_ref (attachment->priv->file_info);
+
+	g_mutex_unlock (&attachment->priv->property_lock);
+
+	return file_info;
 }
 
 void
@@ -1197,44 +1373,54 @@ e_attachment_set_file_info (EAttachment *attachment,
 }
 
 /**
- * e_attachment_get_mime_type:
+ * e_attachment_dup_mime_type:
+ * @attachment: an #EAttachment
  *
- * Returns mime_type part of the file_info as a newly allocated string,
- * which should be freed with g_free().
- * Returns NULL, if mime_type not found or set on the attachment.
+ * Returns the MIME type of @attachment according to its #GFileInfo.
+ * If the @attachment has no #GFileInfo then the function returns %NULL.
+ * Free the returned MIME type string with g_free().
+ *
+ * Returns: a newly-allocated MIME type string, or %NULL
  **/
 gchar *
-e_attachment_get_mime_type (EAttachment *attachment)
+e_attachment_dup_mime_type (EAttachment *attachment)
 {
 	GFileInfo *file_info;
-	const gchar *content_type;
-	gchar *mime_type;
+	const gchar *content_type = NULL;
+	gchar *mime_type = NULL;
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	file_info = e_attachment_get_file_info (attachment);
-	if (file_info == NULL)
-		return NULL;
+	file_info = e_attachment_ref_file_info (attachment);
+	if (file_info != NULL)
+		content_type = g_file_info_get_content_type (file_info);
 
-	content_type = g_file_info_get_content_type (file_info);
-	if (content_type == NULL)
-		return NULL;
+	if (content_type != NULL)
+		mime_type = g_content_type_get_mime_type (content_type);
 
-	mime_type = g_content_type_get_mime_type (content_type);
-	if (!mime_type)
-		return NULL;
+	if (mime_type != NULL)
+		camel_strdown (mime_type);
 
-	camel_strdown (mime_type);
+	g_clear_object (&file_info);
 
 	return mime_type;
 }
 
 GIcon *
-e_attachment_get_icon (EAttachment *attachment)
+e_attachment_ref_icon (EAttachment *attachment)
 {
+	GIcon *icon = NULL;
+
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	return attachment->priv->icon;
+	g_mutex_lock (&attachment->priv->property_lock);
+
+	if (attachment->priv->icon != NULL)
+		icon = g_object_ref (attachment->priv->icon);
+
+	g_mutex_unlock (&attachment->priv->property_lock);
+
+	return icon;
 }
 
 gboolean
@@ -1246,11 +1432,20 @@ e_attachment_get_loading (EAttachment *attachment)
 }
 
 CamelMimePart *
-e_attachment_get_mime_part (EAttachment *attachment)
+e_attachment_ref_mime_part (EAttachment *attachment)
 {
+	CamelMimePart *mime_part = NULL;
+
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	return attachment->priv->mime_part;
+	g_mutex_lock (&attachment->priv->property_lock);
+
+	if (attachment->priv->mime_part != NULL)
+		mime_part = g_object_ref (attachment->priv->mime_part);
+
+	g_mutex_unlock (&attachment->priv->property_lock);
+
+	return mime_part;
 }
 
 void
@@ -1264,11 +1459,13 @@ e_attachment_set_mime_part (EAttachment *attachment,
 		g_object_ref (mime_part);
 	}
 
-	if (attachment->priv->mime_part != NULL)
-		g_object_unref (attachment->priv->mime_part);
+	g_mutex_lock (&attachment->priv->property_lock);
 
+	g_clear_object (&attachment->priv->mime_part);
 	attachment->priv->mime_part = mime_part;
 
+	g_mutex_unlock (&attachment->priv->property_lock);
+
 	g_object_notify (G_OBJECT (attachment), "mime-part");
 }
 
@@ -1372,38 +1569,50 @@ e_attachment_set_signed (EAttachment *attachment,
 	g_object_notify (G_OBJECT (attachment), "signed");
 }
 
-const gchar *
-e_attachment_get_description (EAttachment *attachment)
+gchar *
+e_attachment_dup_description (EAttachment *attachment)
 {
 	GFileInfo *file_info;
 	const gchar *attribute;
+	const gchar *protected;
+	gchar *duplicate;
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
+	file_info = e_attachment_ref_file_info (attachment);
+	if (file_info == NULL)
+		return NULL;
+
 	attribute = G_FILE_ATTRIBUTE_STANDARD_DESCRIPTION;
-	file_info = e_attachment_get_file_info (attachment);
+	protected = g_file_info_get_attribute_string (file_info, attribute);
+	duplicate = g_strdup (protected);
 
-	if (file_info == NULL)
-		return NULL;
+	g_object_unref (file_info);
 
-	return g_file_info_get_attribute_string (file_info, attribute);
+	return duplicate;
 }
 
-const gchar *
-e_attachment_get_thumbnail_path (EAttachment *attachment)
+gchar *
+e_attachment_dup_thumbnail_path (EAttachment *attachment)
 {
 	GFileInfo *file_info;
 	const gchar *attribute;
+	const gchar *protected;
+	gchar *duplicate;
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	attribute = G_FILE_ATTRIBUTE_THUMBNAIL_PATH;
-	file_info = e_attachment_get_file_info (attachment);
-
+	file_info = e_attachment_ref_file_info (attachment);
 	if (file_info == NULL)
 		return NULL;
 
-	return g_file_info_get_attribute_byte_string (file_info, attribute);
+	attribute = G_FILE_ATTRIBUTE_THUMBNAIL_PATH;
+	protected = g_file_info_get_attribute_string (file_info, attribute);
+	duplicate = g_strdup (protected);
+
+	g_object_unref (file_info);
+
+	return duplicate;
 }
 
 gboolean
@@ -1414,8 +1623,10 @@ e_attachment_is_rfc822 (EAttachment *attachment)
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), FALSE);
 
-	mime_type = e_attachment_get_mime_type (attachment);
-	is_rfc822 = mime_type && g_ascii_strcasecmp (mime_type, "message/rfc822") == 0;
+	mime_type = e_attachment_dup_mime_type (attachment);
+	is_rfc822 =
+		(mime_type != NULL) &&
+		(g_ascii_strcasecmp (mime_type, "message/rfc822") == 0);
 	g_free (mime_type);
 
 	return is_rfc822;
@@ -1434,7 +1645,7 @@ e_attachment_list_apps (EAttachment *attachment)
 
 	g_return_val_if_fail (E_IS_ATTACHMENT (attachment), NULL);
 
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 	if (file_info == NULL)
 		return NULL;
 
@@ -1457,6 +1668,8 @@ e_attachment_list_apps (EAttachment *attachment)
 	g_free (allocated);
 
 exit:
+	g_clear_object (&file_info);
+
 	return app_info_list;
 }
 
@@ -1789,12 +2002,13 @@ attachment_load_from_mime_part_thread (GSimpleAsyncResult *simple,
 	CamelStream *null;
 	CamelDataWrapper *dw;
 
-	load_context = g_object_get_data (G_OBJECT (simple), ATTACHMENT_LOAD_CONTEXT);
+	load_context = g_object_get_data (
+		G_OBJECT (simple), ATTACHMENT_LOAD_CONTEXT);
 	g_return_if_fail (load_context != NULL);
 	g_object_set_data (G_OBJECT (simple), ATTACHMENT_LOAD_CONTEXT, NULL);
 
 	attachment = load_context->attachment;
-	mime_part = e_attachment_get_mime_part (attachment);
+	mime_part = e_attachment_ref_mime_part (attachment);
 
 	file_info = g_file_info_new ();
 	load_context->file_info = file_info;
@@ -1849,15 +2063,16 @@ attachment_load_from_mime_part_thread (GSimpleAsyncResult *simple,
 			} else {
 				CamelDataWrapper *content;
 
-				content = camel_medium_get_content (CAMEL_MEDIUM (mime_part));
+				content = camel_medium_get_content (
+					CAMEL_MEDIUM (mime_part));
 				if (CAMEL_IS_MIME_MESSAGE (content))
 					msg = CAMEL_MIME_MESSAGE (content);
 			}
 
-			if (msg)
+			if (msg != NULL)
 				subject = camel_mime_message_get_subject (msg);
 
-			if (subject && *subject)
+			if (subject != NULL && *subject != '\0')
 				string = subject;
 		}
 	} else {
@@ -1883,7 +2098,8 @@ attachment_load_from_mime_part_thread (GSimpleAsyncResult *simple,
 	dw = camel_medium_get_content (CAMEL_MEDIUM (mime_part));
 	null = camel_stream_null_new ();
 	/* this actually downloads the part and makes it available later */
-	camel_data_wrapper_decode_to_stream_sync (dw, null, attachment->priv->cancellable, NULL);
+	camel_data_wrapper_decode_to_stream_sync (
+		dw, null, attachment->priv->cancellable, NULL);
 	g_file_info_set_size (file_info, CAMEL_STREAM_NULL (null)->written);
 	g_object_unref (null);
 
@@ -1896,6 +2112,8 @@ attachment_load_from_mime_part_thread (GSimpleAsyncResult *simple,
 	g_simple_async_result_set_op_res_gpointer (
 		simple, load_context,
 		(GDestroyNotify) attachment_load_context_free);
+
+	g_clear_object (&mime_part);
 }
 
 void
@@ -1926,8 +2144,8 @@ e_attachment_load_async (EAttachment *attachment,
 		return;
 	}
 
-	file = e_attachment_get_file (attachment);
-	mime_part = e_attachment_get_mime_part (attachment);
+	file = e_attachment_ref_file (attachment);
+	mime_part = e_attachment_ref_mime_part (attachment);
 	g_return_if_fail (file != NULL || mime_part != NULL);
 
 	load_context = attachment_load_context_new (
@@ -1944,7 +2162,9 @@ e_attachment_load_async (EAttachment *attachment,
 			attachment_load_query_info_cb, load_context);
 
 	} else if (mime_part != NULL) {
-		g_object_set_data (G_OBJECT (load_context->simple), ATTACHMENT_LOAD_CONTEXT, load_context);
+		g_object_set_data (
+			G_OBJECT (load_context->simple),
+			ATTACHMENT_LOAD_CONTEXT, load_context);
 
 		g_simple_async_result_run_in_thread (
 			load_context->simple,
@@ -1952,6 +2172,9 @@ e_attachment_load_async (EAttachment *attachment,
 			G_PRIORITY_DEFAULT,
 			cancellable);
 	}
+
+	g_clear_object (&file);
+	g_clear_object (&mime_part);
 }
 
 gboolean
@@ -1967,14 +2190,18 @@ e_attachment_load_finish (EAttachment *attachment,
 
 	simple = G_SIMPLE_ASYNC_RESULT (result);
 	load_context = g_simple_async_result_get_op_res_gpointer (simple);
-	if (load_context && load_context->mime_part) {
+
+	if (load_context != NULL && load_context->mime_part != NULL) {
 		const gchar *string;
 
-		string = camel_mime_part_get_disposition (load_context->mime_part);
+		string = camel_mime_part_get_disposition (
+			load_context->mime_part);
 		e_attachment_set_disposition (attachment, string);
 
-		e_attachment_set_file_info (attachment, load_context->file_info);
-		e_attachment_set_mime_part (attachment, load_context->mime_part);
+		e_attachment_set_file_info (
+			attachment, load_context->file_info);
+		e_attachment_set_mime_part (
+			attachment, load_context->mime_part);
 	}
 
 	g_simple_async_result_propagate_error (simple, error);
@@ -2021,7 +2248,7 @@ e_attachment_load_handle_error (EAttachment *attachment,
 		return;
 	}
 
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 
 	if (file_info != NULL)
 		display_name = g_file_info_get_display_name (file_info);
@@ -2035,6 +2262,8 @@ e_attachment_load_handle_error (EAttachment *attachment,
 		primary_text = g_strdup_printf (
 			_("Could not load the attachment"));
 
+	g_clear_object (&file_info);
+
 	dialog = gtk_message_dialog_new_with_markup (
 		parent, GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -2262,8 +2491,8 @@ e_attachment_open_async (EAttachment *attachment,
 
 	g_return_if_fail (E_IS_ATTACHMENT (attachment));
 
-	file = e_attachment_get_file (attachment);
-	mime_part = e_attachment_get_mime_part (attachment);
+	file = e_attachment_ref_file (attachment);
+	mime_part = e_attachment_ref_mime_part (attachment);
 	g_return_if_fail (file != NULL || mime_part != NULL);
 
 	open_context = attachment_open_context_new (
@@ -2280,6 +2509,9 @@ e_attachment_open_async (EAttachment *attachment,
 
 	} else if (mime_part != NULL)
 		attachment_open_save_temporary (open_context);
+
+	g_clear_object (&file);
+	g_clear_object (&mime_part);
 }
 
 gboolean
@@ -2322,7 +2554,7 @@ e_attachment_open_handle_error (EAttachment *attachment,
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		return;
 
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 
 	if (file_info != NULL)
 		display_name = g_file_info_get_display_name (file_info);
@@ -2336,6 +2568,8 @@ e_attachment_open_handle_error (EAttachment *attachment,
 		primary_text = g_strdup_printf (
 			_("Could not open the attachment"));
 
+	g_clear_object (&file_info);
+
 	dialog = gtk_message_dialog_new_with_markup (
 		parent, GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -2468,7 +2702,7 @@ attachment_save_new_candidate (SaveContext *save_context)
 	gchar *basename;
 
 	attachment = save_context->attachment;
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 
 	if (file_info != NULL)
 		display_name = g_file_info_get_display_name (file_info);
@@ -2504,6 +2738,8 @@ attachment_save_new_candidate (SaveContext *save_context)
 
 	g_free (basename);
 
+	g_clear_object (&file_info);
+
 	return candidate;
 }
 
@@ -2618,7 +2854,7 @@ attachment_save_got_output_stream (SaveContext *save_context)
 
 	attachment = save_context->attachment;
 	cancellable = attachment->priv->cancellable;
-	mime_part = e_attachment_get_mime_part (attachment);
+	mime_part = e_attachment_ref_mime_part (attachment);
 
 	/* Decode the MIME part to an in-memory buffer.  We have to do
 	 * this because CamelStream is synchronous-only, and using threads
@@ -2649,6 +2885,8 @@ attachment_save_got_output_stream (SaveContext *save_context)
 		G_PRIORITY_DEFAULT, cancellable,
 		(GAsyncReadyCallback) attachment_save_read_cb,
 		save_context);
+
+	g_clear_object (&mime_part);
 }
 
 static void
@@ -2788,7 +3026,8 @@ e_attachment_save_async (EAttachment *attachment,
 		return;
 	}
 
-	if (e_attachment_get_mime_part (attachment) == NULL) {
+	/* Just peek, don't reference. */
+	if (attachment->priv->mime_part == NULL) {
 		g_simple_async_report_error_in_idle (
 			G_OBJECT (attachment), callback, user_data,
 			G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
@@ -2859,7 +3098,7 @@ e_attachment_save_handle_error (EAttachment *attachment,
 	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		return;
 
-	file_info = e_attachment_get_file_info (attachment);
+	file_info = e_attachment_ref_file_info (attachment);
 
 	if (file_info != NULL)
 		display_name = g_file_info_get_display_name (file_info);
@@ -2873,6 +3112,8 @@ e_attachment_save_handle_error (EAttachment *attachment,
 		primary_text = g_strdup_printf (
 			_("Could not save the attachment"));
 
+	g_clear_object (&file_info);
+
 	dialog = gtk_message_dialog_new_with_markup (
 		parent, GTK_DIALOG_DESTROY_WITH_PARENT,
 		GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
