@@ -1,4 +1,6 @@
 /*
+ * e-tree-model.c
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -12,136 +14,78 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
- *
- * Authors:
- *		Chris Lahey  <clahey@ximian.com>
- *		Chris Toshok <toshok@ximian.com>
- *
- * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
- *
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "e-tree-model.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
-
-#include <gtk/gtk.h>
-#include <libxml/parser.h>
-#include <libxml/xmlmemory.h>
-
-#include "e-marshal.h"
-#include "e-xml-utils.h"
-
-#define ETM_CLASS(e) (E_TREE_MODEL_GET_CLASS(e))
-
-#define d(x)
-
-G_DEFINE_TYPE (ETreeModel, e_tree_model, G_TYPE_OBJECT)
-
 enum {
 	PRE_CHANGE,
-	NO_CHANGE,
 	NODE_CHANGED,
 	NODE_DATA_CHANGED,
-	NODE_COL_CHANGED,
 	NODE_INSERTED,
 	NODE_REMOVED,
 	NODE_DELETED,
-	NODE_REQUEST_COLLAPSE,
 	REBUILT,
 	LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = {0, };
+static guint signals[LAST_SIGNAL];
+
+G_DEFINE_INTERFACE (ETreeModel, e_tree_model, G_TYPE_OBJECT)
 
 static void
-e_tree_model_class_init (ETreeModelClass *class)
+e_tree_model_default_init (ETreeModelInterface *interface)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (class);
-
 	signals[PRE_CHANGE] = g_signal_new (
 		"pre_change",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, pre_change),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0);
-
-	signals[NO_CHANGE] = g_signal_new (
-		"no_change",
-		G_TYPE_FROM_CLASS (object_class),
-		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, no_change),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
+		G_STRUCT_OFFSET (ETreeModelInterface, pre_change),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 0);
 
 	signals[REBUILT] = g_signal_new (
 		"rebuilt",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, rebuilt),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
+		G_STRUCT_OFFSET (ETreeModelInterface, rebuilt),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 0);
 
 	signals[NODE_CHANGED] = g_signal_new (
 		"node_changed",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_changed),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__POINTER,
+		G_STRUCT_OFFSET (ETreeModelInterface, node_changed),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER);
 
 	signals[NODE_DATA_CHANGED] = g_signal_new (
 		"node_data_changed",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_data_changed),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__POINTER,
+		G_STRUCT_OFFSET (ETreeModelInterface, node_data_changed),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER);
 
-	signals[NODE_COL_CHANGED] = g_signal_new (
-		"node_col_changed",
-		G_TYPE_FROM_CLASS (object_class),
-		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_col_changed),
-		(GSignalAccumulator) NULL, NULL,
-		e_marshal_VOID__POINTER_INT,
-		G_TYPE_NONE, 2,
-		G_TYPE_POINTER,
-		G_TYPE_INT);
-
 	signals[NODE_INSERTED] = g_signal_new (
 		"node_inserted",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_inserted),
-		(GSignalAccumulator) NULL, NULL,
-		e_marshal_VOID__POINTER_POINTER,
+		G_STRUCT_OFFSET (ETreeModelInterface, node_inserted),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 2,
 		G_TYPE_POINTER,
 		G_TYPE_POINTER);
 
 	signals[NODE_REMOVED] = g_signal_new (
 		"node_removed",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_removed),
-		(GSignalAccumulator) NULL, NULL,
-		e_marshal_VOID__POINTER_POINTER_INT,
+		G_STRUCT_OFFSET (ETreeModelInterface, node_removed),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 3,
 		G_TYPE_POINTER,
 		G_TYPE_POINTER,
@@ -149,80 +93,14 @@ e_tree_model_class_init (ETreeModelClass *class)
 
 	signals[NODE_DELETED] = g_signal_new (
 		"node_deleted",
-		G_TYPE_FROM_CLASS (object_class),
+		G_TYPE_FROM_INTERFACE (interface),
 		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_deleted),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__POINTER,
+		G_STRUCT_OFFSET (ETreeModelInterface, node_deleted),
+		NULL, NULL, NULL,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER);
-
-	signals[NODE_REQUEST_COLLAPSE] = g_signal_new (
-		"node_request_collapse",
-		G_TYPE_FROM_CLASS (object_class),
-		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (ETreeModelClass, node_request_collapse),
-		(GSignalAccumulator) NULL, NULL,
-		g_cclosure_marshal_VOID__POINTER,
-		G_TYPE_NONE, 1,
-		G_TYPE_POINTER);
-
-	class->get_root              = NULL;
-
-	class->get_parent            = NULL;
-	class->get_first_child       = NULL;
-	class->get_last_child        = NULL;
-	class->get_next              = NULL;
-	class->get_prev              = NULL;
-
-	class->is_root               = NULL;
-	class->is_expandable         = NULL;
-	class->get_children          = NULL;
-	class->depth                 = NULL;
-
-	class->icon_at               = NULL;
-
-	class->get_expanded_default  = NULL;
-	class->column_count          = NULL;
-
-	class->has_save_id           = NULL;
-	class->get_save_id           = NULL;
-	class->has_get_node_by_id    = NULL;
-	class->get_node_by_id        = NULL;
-
-	class->has_change_pending    = NULL;
-
-	class->sort_value_at	  = NULL;
-	class->value_at              = NULL;
-	class->set_value_at          = NULL;
-	class->is_editable           = NULL;
-
-	class->duplicate_value       = NULL;
-	class->free_value            = NULL;
-	class->initialize_value      = NULL;
-	class->value_is_empty        = NULL;
-	class->value_to_string       = NULL;
-
-	class->pre_change            = NULL;
-	class->no_change             = NULL;
-	class->rebuilt		     = NULL;
-	class->node_changed          = NULL;
-	class->node_data_changed     = NULL;
-	class->node_col_changed      = NULL;
-	class->node_inserted         = NULL;
-	class->node_removed          = NULL;
-	class->node_deleted          = NULL;
-	class->node_request_collapse = NULL;
 }
 
-static void
-e_tree_model_init (ETreeModel *tree_model)
-{
-	/* nothing to do */
-}
-
-/* signals */
-
 /**
  * e_tree_model_pre_change:
  * @tree_model:
@@ -237,20 +115,6 @@ e_tree_model_pre_change (ETreeModel *tree_model)
 	g_signal_emit (tree_model, signals[PRE_CHANGE], 0);
 }
 
-/**
- * e_tree_model_no_change:
- * @tree_model:
- *
- * Return value:
- **/
-void
-e_tree_model_no_change (ETreeModel *tree_model)
-{
-	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
-
-	g_signal_emit (tree_model, signals[NO_CHANGE], 0);
-}
-
 /**
  * e_tree_model_rebuilt:
  * @tree_model:
@@ -300,25 +164,6 @@ e_tree_model_node_data_changed (ETreeModel *tree_model,
 	g_signal_emit (tree_model, signals[NODE_DATA_CHANGED], 0, path);
 }
 
-/**
- * e_tree_model_node_col_changed:
- * @tree_model:
- * @path:
- *
- *
- *
- * Return value:
- **/
-void
-e_tree_model_node_col_changed (ETreeModel *tree_model,
-                               ETreePath path,
-                               gint col)
-{
-	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
-
-	g_signal_emit (tree_model, signals[NODE_COL_CHANGED], 0, path, col);
-}
-
 /**
  * e_tree_model_node_inserted:
  * @tree_model:
@@ -376,35 +221,6 @@ e_tree_model_node_deleted (ETreeModel *tree_model,
 	g_signal_emit (tree_model, signals[NODE_DELETED], 0, deleted_path);
 }
 
-/**
- * e_tree_model_node_request_collapse:
- * @tree_model:
- * @collapsed_path:
- *
- *
- **/
-void
-e_tree_model_node_request_collapse (ETreeModel *tree_model,
-                                    ETreePath collapsed_path)
-{
-	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
-
-	g_signal_emit (tree_model, signals[NODE_REQUEST_COLLAPSE], 0, collapsed_path);
-}
-
-/**
- * e_tree_model_new
- *
- * XXX docs here.
- *
- * return values: a newly constructed ETreeModel.
- */
-ETreeModel *
-e_tree_model_new (void)
-{
-	return g_object_new (E_TYPE_TREE_MODEL, NULL);
-}
-
 /**
  * e_tree_model_get_root
  * @tree_model: the ETreeModel of which we want the root node.
@@ -416,12 +232,14 @@ e_tree_model_new (void)
 ETreePath
 e_tree_model_get_root (ETreeModel *tree_model)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_root)
-		return ETM_CLASS (tree_model)->get_root (tree_model);
-	else
-		return NULL;
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_root != NULL, NULL);
+
+	return interface->get_root (tree_model);
 }
 
 /**
@@ -437,12 +255,14 @@ ETreePath
 e_tree_model_node_get_parent (ETreeModel *tree_model,
                               ETreePath path)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_parent)
-		return ETM_CLASS (tree_model)->get_parent (tree_model, path);
-	else
-		return NULL;
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_parent != NULL, NULL);
+
+	return interface->get_parent (tree_model, path);
 }
 
 /**
@@ -458,33 +278,14 @@ ETreePath
 e_tree_model_node_get_first_child (ETreeModel *tree_model,
                                    ETreePath path)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_first_child)
-		return ETM_CLASS (tree_model)->get_first_child (tree_model, path);
-	else
-		return NULL;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_first_child != NULL, NULL);
 
-/**
- * e_tree_model_node_get_last_child:
- * @tree_model:
- * @path:
- *
- *
- *
- * Return value:
- **/
-ETreePath
-e_tree_model_node_get_last_child (ETreeModel *tree_model,
-                                  ETreePath path)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
-
-	if (ETM_CLASS (tree_model)->get_last_child)
-		return ETM_CLASS (tree_model)->get_last_child (tree_model, path);
-	else
-		return NULL;
+	return interface->get_first_child (tree_model, path);
 }
 
 /**
@@ -500,33 +301,14 @@ ETreePath
 e_tree_model_node_get_next (ETreeModel *tree_model,
                             ETreePath path)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_next)
-		return ETM_CLASS (tree_model)->get_next (tree_model, path);
-	else
-		return NULL;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_next != NULL, NULL);
 
-/**
- * e_tree_model_node_get_prev:
- * @tree_model:
- * @path:
- *
- *
- *
- * Return value:
- **/
-ETreePath
-e_tree_model_node_get_prev (ETreeModel *tree_model,
-                            ETreePath path)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
-
-	if (ETM_CLASS (tree_model)->get_prev)
-		return ETM_CLASS (tree_model)->get_prev (tree_model, path);
-	else
-		return NULL;
+	return interface->get_next (tree_model, path);
 }
 
 /**
@@ -542,12 +324,14 @@ gboolean
 e_tree_model_node_is_root (ETreeModel *tree_model,
                            ETreePath path)
 {
-	g_return_val_if_fail (tree_model != NULL, FALSE);
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->is_root)
-		return ETM_CLASS (tree_model)->is_root (tree_model, path);
-	else
-		return FALSE;
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->is_root != NULL, FALSE);
+
+	return interface->is_root (tree_model, path);
 }
 
 /**
@@ -563,25 +347,29 @@ gboolean
 e_tree_model_node_is_expandable (ETreeModel *tree_model,
                                  ETreePath path)
 {
-	g_return_val_if_fail (tree_model != NULL, FALSE);
+	ETreeModelInterface *interface;
+
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
 	g_return_val_if_fail (path != NULL, FALSE);
 
-	if (ETM_CLASS (tree_model)->is_expandable)
-		return ETM_CLASS (tree_model)->is_expandable (tree_model, path);
-	else
-		return FALSE;
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->is_expandable != NULL, FALSE);
+
+	return interface->is_expandable (tree_model, path);
 }
 
 guint
-e_tree_model_node_get_children (ETreeModel *tree_model,
-                                ETreePath path,
-                                ETreePath **paths)
+e_tree_model_node_get_n_children (ETreeModel *tree_model,
+                                  ETreePath path)
 {
-	g_return_val_if_fail (tree_model != NULL, 0);
-	if (ETM_CLASS (tree_model)->get_children)
-		return ETM_CLASS (tree_model)->get_children (tree_model, path, paths);
-	else
-		return 0;
+	ETreeModelInterface *interface;
+
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), 0);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_n_children != NULL, 0);
+
+	return interface->get_n_children (tree_model, path);
 }
 
 /**
@@ -597,33 +385,14 @@ guint
 e_tree_model_node_depth (ETreeModel *tree_model,
                          ETreePath path)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), 0);
 
-	if (ETM_CLASS (tree_model)->depth)
-		return ETM_CLASS (tree_model)->depth (tree_model, path);
-	else
-		return 0;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->depth != NULL, 0);
 
-/**
- * e_tree_model_icon_at
- * @tree_model: The ETreeModel.
- * @path: The ETreePath to the node we're getting the icon of.
- *
- * XXX docs here.
- *
- * return values: the GdkPixbuf associated with this node.
- */
-GdkPixbuf *
-e_tree_model_icon_at (ETreeModel *tree_model,
-                      ETreePath path)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
-
-	if (ETM_CLASS (tree_model)->icon_at)
-		return ETM_CLASS (tree_model)->icon_at (tree_model, path);
-	else
-		return NULL;
+	return interface->depth (tree_model, path);
 }
 
 /**
@@ -637,12 +406,14 @@ e_tree_model_icon_at (ETreeModel *tree_model,
 gboolean
 e_tree_model_get_expanded_default (ETreeModel *tree_model)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
 
-	if (ETM_CLASS (tree_model)->get_expanded_default)
-		return ETM_CLASS (tree_model)->get_expanded_default (tree_model);
-	else
-		return FALSE;
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_expanded_default != NULL, FALSE);
+
+	return interface->get_expanded_default (tree_model);
 }
 
 /**
@@ -656,31 +427,14 @@ e_tree_model_get_expanded_default (ETreeModel *tree_model)
 gint
 e_tree_model_column_count (ETreeModel *tree_model)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), 0);
 
-	if (ETM_CLASS (tree_model)->column_count)
-		return ETM_CLASS (tree_model)->column_count (tree_model);
-	else
-		return 0;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->column_count != NULL, 0);
 
-/**
- * e_tree_model_has_save_id
- * @tree_model: The ETreeModel.
- *
- * XXX docs here.
- *
- * return values: Whether this tree has valid save id data.
- */
-gboolean
-e_tree_model_has_save_id (ETreeModel *tree_model)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
-
-	if (ETM_CLASS (tree_model)->has_save_id)
-		return ETM_CLASS (tree_model)->has_save_id (tree_model);
-	else
-		return FALSE;
+	return interface->column_count (tree_model);
 }
 
 /**
@@ -696,31 +450,14 @@ gchar *
 e_tree_model_get_save_id (ETreeModel *tree_model,
                           ETreePath path)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_save_id)
-		return ETM_CLASS (tree_model)->get_save_id (tree_model, path);
-	else
-		return NULL;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_save_id != NULL, NULL);
 
-/**
- * e_tree_model_has_get_node_by_id
- * @tree_model: The ETreeModel.
- *
- * XXX docs here.
- *
- * return values: Whether this tree can quickly get a node from its save id.
- */
-gboolean
-e_tree_model_has_get_node_by_id (ETreeModel *tree_model)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
-
-	if (ETM_CLASS (tree_model)->has_get_node_by_id)
-		return ETM_CLASS (tree_model)->has_get_node_by_id (tree_model);
-	else
-		return FALSE;
+	return interface->get_save_id (tree_model, path);
 }
 
 /**
@@ -739,31 +476,14 @@ ETreePath
 e_tree_model_get_node_by_id (ETreeModel *tree_model,
                              const gchar *save_id)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->get_node_by_id)
-		return ETM_CLASS (tree_model)->get_node_by_id (tree_model, save_id);
-	else
-		return NULL;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->get_node_by_id != NULL, NULL);
 
-/**
- * e_tree_model_has_change_pending
- * @tree_model: The ETreeModel.
- *
- * XXX docs here.
- *
- * return values: Whether this tree has valid save id data.
- */
-gboolean
-e_tree_model_has_change_pending (ETreeModel *tree_model)
-{
-	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), FALSE);
-
-	if (ETM_CLASS (tree_model)->has_change_pending)
-		return ETM_CLASS (tree_model)->has_change_pending (tree_model);
-	else
-		return FALSE;
+	return interface->get_node_by_id (tree_model, save_id);
 }
 
 /**
@@ -791,12 +511,14 @@ e_tree_model_sort_value_at (ETreeModel *tree_model,
                             ETreePath path,
                             gint col)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->sort_value_at)
-		return ETM_CLASS (tree_model)->sort_value_at (tree_model, path, col);
-	else
-		return NULL;
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->sort_value_at != NULL, NULL);
+
+	return interface->sort_value_at (tree_model, path, col);
 }
 
 /**
@@ -823,46 +545,14 @@ e_tree_model_value_at (ETreeModel *tree_model,
                        ETreePath path,
                        gint col)
 {
+	ETreeModelInterface *interface;
+
 	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
 
-	if (ETM_CLASS (tree_model)->value_at)
-		return ETM_CLASS (tree_model)->value_at (tree_model, path, col);
-	else
-		return NULL;
-}
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->value_at != NULL, NULL);
 
-void
-e_tree_model_set_value_at (ETreeModel *tree_model,
-                           ETreePath path,
-                           gint col,
-                           gconstpointer val)
-{
-	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
-
-	if (ETM_CLASS (tree_model)->set_value_at)
-		ETM_CLASS (tree_model)->set_value_at (tree_model, path, col, val);
-}
-
-/**
- * e_tree_model_node_is_editable:
- * @tree_model:
- * @path:
- *
- *
- *
- * Return value:
- **/
-gboolean
-e_tree_model_node_is_editable (ETreeModel *tree_model,
-                               ETreePath path,
-                               gint col)
-{
-	g_return_val_if_fail (tree_model != NULL, FALSE);
-
-	if (ETM_CLASS (tree_model)->is_editable)
-		return ETM_CLASS (tree_model)->is_editable (tree_model, path, col);
-	else
-		return FALSE;
+	return interface->value_at (tree_model, path, col);
 }
 
 /**
@@ -879,12 +569,14 @@ e_tree_model_duplicate_value (ETreeModel *tree_model,
                               gint col,
                               gconstpointer value)
 {
-	g_return_val_if_fail (tree_model != NULL, NULL);
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->duplicate_value)
-		return ETM_CLASS (tree_model)->duplicate_value (tree_model, col, value);
-	else
-		return NULL;
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->duplicate_value != NULL, NULL);
+
+	return interface->duplicate_value (tree_model, col, value);
 }
 
 /**
@@ -901,10 +593,14 @@ e_tree_model_free_value (ETreeModel *tree_model,
                          gint col,
                          gpointer value)
 {
-	g_return_if_fail (tree_model != NULL);
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->free_value)
-		ETM_CLASS (tree_model)->free_value (tree_model, col, value);
+	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_if_fail (interface->free_value != NULL);
+
+	interface->free_value (tree_model, col, value);
 }
 
 /**
@@ -920,12 +616,14 @@ gpointer
 e_tree_model_initialize_value (ETreeModel *tree_model,
                                gint col)
 {
-	g_return_val_if_fail (tree_model != NULL, NULL);
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->initialize_value)
-		return ETM_CLASS (tree_model)->initialize_value (tree_model, col);
-	else
-		return NULL;
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->initialize_value != NULL, NULL);
+
+	return interface->initialize_value (tree_model, col);
 }
 
 /**
@@ -942,12 +640,14 @@ e_tree_model_value_is_empty (ETreeModel *tree_model,
                              gint col,
                              gconstpointer value)
 {
-	g_return_val_if_fail (tree_model != NULL, TRUE);
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->value_is_empty)
-		return ETM_CLASS (tree_model)->value_is_empty (tree_model, col, value);
-	else
-		return TRUE;
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), TRUE);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->value_is_empty != NULL, TRUE);
+
+	return interface->value_is_empty (tree_model, col, value);
 }
 
 /**
@@ -964,12 +664,14 @@ e_tree_model_value_to_string (ETreeModel *tree_model,
                               gint col,
                               gconstpointer value)
 {
-	g_return_val_if_fail (tree_model != NULL, g_strdup (""));
+	ETreeModelInterface *interface;
 
-	if (ETM_CLASS (tree_model)->value_to_string)
-		return ETM_CLASS (tree_model)->value_to_string (tree_model, col, value);
-	else
-		return g_strdup ("");
+	g_return_val_if_fail (E_IS_TREE_MODEL (tree_model), NULL);
+
+	interface = E_TREE_MODEL_GET_INTERFACE (tree_model);
+	g_return_val_if_fail (interface->value_to_string != NULL, NULL);
+
+	return interface->value_to_string (tree_model, col, value);
 }
 
 /**
@@ -1006,46 +708,10 @@ e_tree_model_node_traverse (ETreeModel *tree_model,
 	}
 }
 
-/**
- * e_tree_model_node_traverse_preorder:
- * @tree_model:
- * @path:
- * @func:
- * @data:
- *
- *
- **/
-void
-e_tree_model_node_traverse_preorder (ETreeModel *tree_model,
-                                     ETreePath path,
-                                     ETreePathFunc func,
-                                     gpointer data)
-{
-	ETreePath child;
-
-	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
-	g_return_if_fail (path != NULL);
-
-	child = e_tree_model_node_get_first_child (tree_model, path);
-
-	while (child) {
-		ETreePath next_child;
-
-		if (func (tree_model, child, data))
-			return;
-
-		next_child = e_tree_model_node_get_next (tree_model, child);
-		e_tree_model_node_traverse_preorder (tree_model, child, func, data);
-
-		child = next_child;
-	}
-}
-
 static ETreePath
 e_tree_model_node_real_traverse (ETreeModel *model,
                                  ETreePath path,
                                  ETreePath end_path,
-                                 gboolean forward_direction,
                                  ETreePathFunc func,
                                  gpointer data)
 {
@@ -1054,30 +720,21 @@ e_tree_model_node_real_traverse (ETreeModel *model,
 	g_return_val_if_fail (E_IS_TREE_MODEL (model), NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 
-	if (forward_direction)
-		child = e_tree_model_node_get_first_child (model, path);
-	else
-		child = e_tree_model_node_get_last_child (model, path);
+	child = e_tree_model_node_get_first_child (model, path);
 
 	while (child) {
 		ETreePath result;
 
-		if (forward_direction && (child == end_path || func (model, child, data)))
+		if (child == end_path || func (model, child, data))
 			return child;
 
 		if ((result = e_tree_model_node_real_traverse (
-			model, child, end_path,
-			forward_direction, func, data)))
+			model, child, end_path, func, data)))
 			return result;
 
-		if (!forward_direction && (child == end_path || func (model, child, data)))
-			return child;
-
-		if (forward_direction)
-			child = e_tree_model_node_get_next (model, child);
-		else
-			child = e_tree_model_node_get_prev (model, child);
+		child = e_tree_model_node_get_next (model, child);
 	}
+
 	return NULL;
 }
 
@@ -1086,7 +743,6 @@ e_tree_model_node_real_traverse (ETreeModel *model,
  * @tree_model:
  * @path:
  * @end_path:
- * @forward_direction:
  * @func:
  * @data:
  *
@@ -1096,7 +752,6 @@ ETreePath
 e_tree_model_node_find (ETreeModel *tree_model,
                         ETreePath path,
                         ETreePath end_path,
-                        gboolean forward_direction,
                         ETreePathFunc func,
                         gpointer data)
 {
@@ -1110,35 +765,23 @@ e_tree_model_node_find (ETreeModel *tree_model,
 		ETreePath root;
 		root = e_tree_model_get_root (tree_model);
 
-		if (forward_direction && (end_path == root || func (tree_model, root, data)))
+		if (end_path == root || func (tree_model, root, data))
 			return root;
 
 		result = e_tree_model_node_real_traverse (
-			tree_model, root, end_path, forward_direction, func, data);
+			tree_model, root, end_path, func, data);
 		if (result)
 			return result;
 
-		if (!forward_direction && (end_path == root || func (tree_model, root, data)))
-			return root;
-
 		return NULL;
 	}
 
 	while (1) {
 
-		if (forward_direction) {
-			if ((result = e_tree_model_node_real_traverse (
-				tree_model, path, end_path,
-				forward_direction, func, data)))
-				return result;
-			next = e_tree_model_node_get_next (tree_model, path);
-		} else {
-			next = e_tree_model_node_get_prev (tree_model, path);
-			if (next && (result = e_tree_model_node_real_traverse (
-				tree_model, next, end_path,
-				forward_direction, func, data)))
-				return result;
-		}
+		if ((result = e_tree_model_node_real_traverse (
+			tree_model, path, end_path, func, data)))
+			return result;
+		next = e_tree_model_node_get_next (tree_model, path);
 
 		while (next == NULL) {
 			path = e_tree_model_node_get_parent (tree_model, path);
@@ -1146,10 +789,7 @@ e_tree_model_node_find (ETreeModel *tree_model,
 			if (path == NULL)
 				return NULL;
 
-			if (forward_direction)
-				next = e_tree_model_node_get_next (tree_model, path);
-			else
-				next = path;
+			next = e_tree_model_node_get_next (tree_model, path);
 		}
 
 		if (end_path == next || func (tree_model, next, data))
