@@ -1,4 +1,5 @@
 /*
+ * e-tree-model.h
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -13,21 +14,14 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
- *
- * Authors:
- *		Chris Lahey <clahey@ximian.com>
- *		Chris Toshok <toshok@ximian.com>
- *
- * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
- *
  */
 
 #if !defined (__E_UTIL_H_INSIDE__) && !defined (LIBEUTIL_COMPILATION)
 #error "Only <e-util/e-util.h> should be included directly."
 #endif
 
-#ifndef _E_TREE_MODEL_H_
-#define _E_TREE_MODEL_H_
+#ifndef E_TREE_MODEL_H
+#define E_TREE_MODEL_H
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -37,79 +31,54 @@
 #define E_TREE_MODEL(obj) \
 	(G_TYPE_CHECK_INSTANCE_CAST \
 	((obj), E_TYPE_TREE_MODEL, ETreeModel))
-#define E_TREE_MODEL_CLASS(cls) \
-	(G_TYPE_CHECK_CLASS_CAST \
-	((cls), E_TYPE_TREE_MODEL, ETreeModelClass))
 #define E_IS_TREE_MODEL(obj) \
 	(G_TYPE_CHECK_INSTANCE_TYPE \
 	((obj), E_TYPE_TREE_MODEL))
-#define E_IS_TREE_MODEL_CLASS(cls) \
-	(G_TYPE_CHECK_CLASS_TYPE \
-	((cls), E_TYPE_TREE_MODEL))
-#define E_TREE_MODEL_GET_CLASS(obj) \
-	(G_TYPE_INSTANCE_GET_CLASS \
-	((obj), E_TYPE_TREE_MODEL, ETreeModelClass))
+#define E_TREE_MODEL_GET_INTERFACE(obj) \
+	(G_TYPE_INSTANCE_GET_INTERFACE \
+	((obj), E_TYPE_TREE_MODEL, ETreeModelInterface))
 
 G_BEGIN_DECLS
 
 typedef gpointer ETreePath;
 
 typedef struct _ETreeModel ETreeModel;
-typedef struct _ETreeModelClass ETreeModelClass;
+typedef struct _ETreeModelInterface ETreeModelInterface;
 
 typedef gboolean	(*ETreePathFunc)	(ETreeModel *tree_model,
 						 ETreePath path,
 						 gpointer data);
 
-struct _ETreeModel {
-	GObject parent;
-};
+struct _ETreeModelInterface {
+	GTypeInterface parent_class;
 
-struct _ETreeModelClass {
-	GObjectClass parent_class;
-
-	/*
-	 * Virtual methods
-	 */
 	ETreePath	(*get_root)		(ETreeModel *tree_model);
 
 	ETreePath	(*get_parent)		(ETreeModel *tree_model,
 						 ETreePath path);
 	ETreePath	(*get_first_child)	(ETreeModel *tree_model,
 						 ETreePath path);
-	ETreePath	(*get_last_child)	(ETreeModel *tree_model,
-						 ETreePath path);
 	ETreePath	(*get_next)		(ETreeModel *tree_model,
 						 ETreePath path);
-	ETreePath	(*get_prev)		(ETreeModel *tree_model,
-						 ETreePath path);
 
 	gboolean	(*is_root)		(ETreeModel *tree_model,
 						 ETreePath path);
 	gboolean	(*is_expandable)	(ETreeModel *tree_model,
 						 ETreePath path);
-	guint		(*get_children)		(ETreeModel *tree_model,
-						 ETreePath path,
-						 ETreePath **paths);
-	guint		(*depth)		(ETreeModel *tree_model,
+	guint		(*get_n_children)	(ETreeModel *tree_model,
 						 ETreePath path);
-
-	GdkPixbuf *	(*icon_at)		(ETreeModel *tree_model,
+	guint		(*depth)		(ETreeModel *tree_model,
 						 ETreePath path);
 
 	gboolean	(*get_expanded_default)	(ETreeModel *tree_model);
 	gint		(*column_count)		(ETreeModel *tree_model);
 
-	gboolean	(*has_save_id)		(ETreeModel *tree_model);
 	gchar *		(*get_save_id)		(ETreeModel *tree_model,
 						 ETreePath path);
 
-	gboolean	(*has_get_node_by_id)	(ETreeModel *tree_model);
 	ETreePath	(*get_node_by_id)	(ETreeModel *tree_model,
 						 const gchar *save_id);
 
-	gboolean	(*has_change_pending)	(ETreeModel *tree_model);
-
 	/*
 	 * ETable analogs
 	 */
@@ -119,13 +88,6 @@ struct _ETreeModelClass {
 	gpointer	(*value_at)		(ETreeModel *tree_model,
 						 ETreePath path,
 						 gint col);
-	void		(*set_value_at)		(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col,
-						 gconstpointer val);
-	gboolean	(*is_editable)		(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col);
 
 	gpointer	(*duplicate_value)	(ETreeModel *tree_model,
 						 gint col,
@@ -152,14 +114,10 @@ struct _ETreeModelClass {
 	 */
 
 	void		(*pre_change)		(ETreeModel *tree_model);
-	void		(*no_change)		(ETreeModel *tree_model);
 	void		(*node_changed)		(ETreeModel *tree_model,
 						 ETreePath path);
 	void		(*node_data_changed)	(ETreeModel *tree_model,
 						 ETreePath path);
-	void		(*node_col_changed)	(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col);
 	void		(*node_inserted)	(ETreeModel *tree_model,
 						 ETreePath parent,
 						 ETreePath inserted_path);
@@ -170,17 +128,9 @@ struct _ETreeModelClass {
 	void		(*node_deleted)		(ETreeModel *tree_model,
 						 ETreePath deleted_path);
 	void		(*rebuilt)		(ETreeModel *tree_model);
-
-	/* This signal requests that any viewers of the tree that
-	 * collapse and expand nodes collapse this node.
-	 */
-	void		(*node_request_collapse)
-						(ETreeModel *tree_model,
-						 ETreePath path);
 };
 
 GType		e_tree_model_get_type		(void) G_GNUC_CONST;
-ETreeModel *	e_tree_model_new		(void);
 
 /* tree traversal operations */
 ETreePath	e_tree_model_get_root		(ETreeModel *tree_model);
@@ -189,49 +139,32 @@ ETreePath	e_tree_model_node_get_parent	(ETreeModel *tree_model,
 ETreePath	e_tree_model_node_get_first_child
 						(ETreeModel *tree_model,
 						 ETreePath path);
-ETreePath	e_tree_model_node_get_last_child
-						(ETreeModel *tree_model,
-						 ETreePath path);
 ETreePath	e_tree_model_node_get_next	(ETreeModel *tree_model,
 						 ETreePath path);
-ETreePath	e_tree_model_node_get_prev	(ETreeModel *tree_model,
-						 ETreePath path);
 
 /* node accessors */
 gboolean	e_tree_model_node_is_root	(ETreeModel *tree_model,
 						 ETreePath path);
 gboolean	e_tree_model_node_is_expandable	(ETreeModel *tree_model,
 						 ETreePath path);
-guint		e_tree_model_node_get_children	(ETreeModel *tree_model,
-						 ETreePath path,
-						 ETreePath **paths);
-guint		e_tree_model_node_depth		(ETreeModel *tree_model,
+guint		e_tree_model_node_get_n_children
+						(ETreeModel *tree_model,
 						 ETreePath path);
-GdkPixbuf *	e_tree_model_icon_at		(ETreeModel *tree_model,
+guint		e_tree_model_node_depth		(ETreeModel *tree_model,
 						 ETreePath path);
 gboolean	e_tree_model_get_expanded_default
 						(ETreeModel *tree_model);
 gint		e_tree_model_column_count	(ETreeModel *tree_model);
-gboolean	e_tree_model_has_save_id	(ETreeModel *tree_model);
 gchar *		e_tree_model_get_save_id	(ETreeModel *tree_model,
 						 ETreePath path);
-gboolean	e_tree_model_has_get_node_by_id	(ETreeModel *tree_model);
 ETreePath	e_tree_model_get_node_by_id	(ETreeModel *tree_model,
 						 const gchar *save_id);
-gboolean	e_tree_model_has_change_pending	(ETreeModel *tree_model);
 gpointer	e_tree_model_sort_value_at	(ETreeModel *tree_model,
 						 ETreePath path,
 						 gint col);
 gpointer	e_tree_model_value_at		(ETreeModel *tree_model,
 						 ETreePath path,
 						 gint col);
-void		e_tree_model_set_value_at	(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col,
-						 gconstpointer val);
-gboolean	e_tree_model_node_is_editable	(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col);
 gpointer	e_tree_model_duplicate_value	(ETreeModel *tree_model,
 						 gint col,
 						 gconstpointer value);
@@ -252,15 +185,9 @@ void		e_tree_model_node_traverse	(ETreeModel *tree_model,
 						 ETreePath path,
 						 ETreePathFunc func,
 						 gpointer data);
-void		e_tree_model_node_traverse_preorder
-						(ETreeModel *tree_model,
-						 ETreePath path,
-						 ETreePathFunc func,
-						 gpointer data);
 ETreePath	e_tree_model_node_find		(ETreeModel *tree_model,
 						 ETreePath path,
 						 ETreePath end_path,
-						 gboolean forward_direction,
 						 ETreePathFunc func,
 						 gpointer data);
 
@@ -268,15 +195,11 @@ ETreePath	e_tree_model_node_find		(ETreeModel *tree_model,
 ** Routines for emitting signals on the ETreeModel
 */
 void		e_tree_model_pre_change		(ETreeModel *tree_model);
-void		e_tree_model_no_change		(ETreeModel *tree_model);
 void		e_tree_model_rebuilt		(ETreeModel *tree_model);
 void		e_tree_model_node_changed	(ETreeModel *tree_model,
 						 ETreePath path);
 void		e_tree_model_node_data_changed	(ETreeModel *tree_model,
 						 ETreePath path);
-void		e_tree_model_node_col_changed	(ETreeModel *tree_model,
-						 ETreePath path,
-						 gint col);
 void		e_tree_model_node_inserted	(ETreeModel *tree_model,
 						 ETreePath parent_path,
 						 ETreePath inserted_path);
@@ -286,10 +209,7 @@ void		e_tree_model_node_removed	(ETreeModel *tree_model,
 						 gint old_position);
 void		e_tree_model_node_deleted	(ETreeModel *tree_model,
 						 ETreePath deleted_path);
-void		e_tree_model_node_request_collapse
-						(ETreeModel *tree_model,
-						 ETreePath collapsed_path);
 
 G_END_DECLS
 
-#endif /* _E_TREE_MODEL_H */
+#endif /* E_TREE_MODEL_H */
