@@ -57,10 +57,8 @@ struct _ETreeSelectionModelPrivate {
 	gchar *cursor_save_id;
 
 	gint tree_model_pre_change_id;
-	gint tree_model_no_change_id;
 	gint tree_model_node_changed_id;
 	gint tree_model_node_data_changed_id;
-	gint tree_model_node_col_changed_id;
 	gint tree_model_node_inserted_id;
 	gint tree_model_node_removed_id;
 	gint tree_model_node_deleted_id;
@@ -175,19 +173,9 @@ etsm_pre_change (ETreeModel *etm,
 	g_free (etsm->priv->cursor_save_id);
 	etsm->priv->cursor_save_id = NULL;
 
-	if (e_tree_model_has_get_node_by_id (etm) &&
-	    e_tree_model_has_save_id (etm) &&
-	    etsm->priv->cursor_path) {
+	if (etsm->priv->cursor_path != NULL)
 		etsm->priv->cursor_save_id = e_tree_model_get_save_id (
 			etm, etsm->priv->cursor_path);
-	}
-}
-
-static void
-etsm_no_change (ETreeModel *etm,
-                ETreeSelectionModel *etsm)
-{
-	free_id (etsm);
 }
 
 static void
@@ -206,15 +194,6 @@ etsm_node_data_changed (ETreeModel *etm,
 	free_id (etsm);
 }
 
-static void
-etsm_node_col_changed (ETreeModel *etm,
-                       ETreePath node,
-                       gint col,
-                       ETreeSelectionModel *etsm)
-{
-	free_id (etsm);
-}
-
 static void
 etsm_node_inserted (ETreeModel *etm,
                     ETreePath parent,
@@ -259,10 +238,6 @@ add_model (ETreeSelectionModel *etsm,
 		priv->model, "pre_change",
 		G_CALLBACK (etsm_pre_change), etsm);
 
-	priv->tree_model_no_change_id = g_signal_connect_after (
-		priv->model, "no_change",
-		G_CALLBACK (etsm_no_change), etsm);
-
 	priv->tree_model_node_changed_id = g_signal_connect_after (
 		priv->model, "node_changed",
 		G_CALLBACK (etsm_node_changed), etsm);
@@ -271,10 +246,6 @@ add_model (ETreeSelectionModel *etsm,
 		priv->model, "node_data_changed",
 		G_CALLBACK (etsm_node_data_changed), etsm);
 
-	priv->tree_model_node_col_changed_id = g_signal_connect_after (
-		priv->model, "node_col_changed",
-		G_CALLBACK (etsm_node_col_changed), etsm);
-
 	priv->tree_model_node_inserted_id = g_signal_connect_after (
 		priv->model, "node_inserted",
 		G_CALLBACK (etsm_node_inserted), etsm);
@@ -298,14 +269,10 @@ drop_model (ETreeSelectionModel *etsm)
 
 	g_signal_handler_disconnect (
 		priv->model, priv->tree_model_pre_change_id);
-	g_signal_handler_disconnect (
-		priv->model, priv->tree_model_no_change_id);
 	g_signal_handler_disconnect (
 		priv->model, priv->tree_model_node_changed_id);
 	g_signal_handler_disconnect (
 		priv->model, priv->tree_model_node_data_changed_id);
-	g_signal_handler_disconnect (
-		priv->model, priv->tree_model_node_col_changed_id);
 	g_signal_handler_disconnect (
 		priv->model, priv->tree_model_node_inserted_id);
 	g_signal_handler_disconnect (
@@ -317,10 +284,8 @@ drop_model (ETreeSelectionModel *etsm)
 	priv->model = NULL;
 
 	priv->tree_model_pre_change_id = 0;
-	priv->tree_model_no_change_id = 0;
 	priv->tree_model_node_changed_id = 0;
 	priv->tree_model_node_data_changed_id = 0;
-	priv->tree_model_node_col_changed_id = 0;
 	priv->tree_model_node_inserted_id = 0;
 	priv->tree_model_node_removed_id = 0;
 	priv->tree_model_node_deleted_id = 0;
