@@ -86,14 +86,11 @@ struct _ETreeTableAdapterPrivate {
 	gint          last_access;
 
 	gint          pre_change_id;
-	gint          no_change_id;
 	gint	     rebuilt_id;
 	gint          node_changed_id;
 	gint          node_data_changed_id;
-	gint          node_col_changed_id;
 	gint          node_inserted_id;
 	gint          node_removed_id;
-	gint          node_request_collapse_id;
 	gint          sort_info_changed_id;
 
 	guint        resort_idle_id;
@@ -583,22 +580,16 @@ etta_dispose (GObject *object)
 	if (priv->source) {
 		g_signal_handler_disconnect (
 			priv->source, priv->pre_change_id);
-		g_signal_handler_disconnect (
-			priv->source, priv->no_change_id);
 		g_signal_handler_disconnect (
 			priv->source, priv->rebuilt_id);
 		g_signal_handler_disconnect (
 			priv->source, priv->node_changed_id);
 		g_signal_handler_disconnect (
 			priv->source, priv->node_data_changed_id);
-		g_signal_handler_disconnect (
-			priv->source, priv->node_col_changed_id);
 		g_signal_handler_disconnect (
 			priv->source, priv->node_inserted_id);
 		g_signal_handler_disconnect (
 			priv->source, priv->node_removed_id);
-		g_signal_handler_disconnect (
-			priv->source, priv->node_request_collapse_id);
 
 		g_object_unref (priv->source);
 		priv->source = NULL;
@@ -619,9 +610,7 @@ etta_column_count (ETableModel *etm)
 static gboolean
 etta_has_save_id (ETableModel *etm)
 {
-	ETreeTableAdapter *etta = (ETreeTableAdapter *) etm;
-
-	return e_tree_model_has_save_id (etta->priv->source);
+	return TRUE;
 }
 
 static gchar *
@@ -633,14 +622,6 @@ etta_get_save_id (ETableModel *etm,
 	return e_tree_model_get_save_id (etta->priv->source, get_path (etta, row));
 }
 
-static gboolean
-etta_has_change_pending (ETableModel *etm)
-{
-	ETreeTableAdapter *etta = (ETreeTableAdapter *) etm;
-
-	return e_tree_model_has_change_pending (etta->priv->source);
-}
-
 static gint
 etta_row_count (ETableModel *etm)
 {
@@ -676,9 +657,7 @@ etta_set_value_at (ETableModel *etm,
                    gint row,
                    gconstpointer val)
 {
-	ETreeTableAdapter *etta = (ETreeTableAdapter *) etm;
-
-	e_tree_model_set_value_at (etta->priv->source, get_path (etta, row), col, val);
+	g_warn_if_reached ();
 }
 
 static gboolean
@@ -686,9 +665,7 @@ etta_is_cell_editable (ETableModel *etm,
                        gint col,
                        gint row)
 {
-	ETreeTableAdapter *etta = (ETreeTableAdapter *) etm;
-
-	return e_tree_model_node_is_editable (etta->priv->source, get_path (etta, row), col);
+	return FALSE;
 }
 
 static void
@@ -771,7 +748,6 @@ etta_class_init (ETreeTableAdapterClass *class)
 	table_model_class->has_save_id = etta_has_save_id;
 	table_model_class->get_save_id = etta_get_save_id;
 
-	table_model_class->has_change_pending = etta_has_change_pending;
 	table_model_class->duplicate_value = etta_duplicate_value;
 	table_model_class->free_value = etta_free_value;
 	table_model_class->initialize_value = etta_initialize_value;
@@ -807,13 +783,6 @@ etta_proxy_pre_change (ETreeModel *etm,
 	e_table_model_pre_change (E_TABLE_MODEL (etta));
 }
 
-static void
-etta_proxy_no_change (ETreeModel *etm,
-                      ETreeTableAdapter *etta)
-{
-	e_table_model_no_change (E_TABLE_MODEL (etta));
-}
-
 static void
 etta_proxy_rebuilt (ETreeModel *etm,
                     ETreeTableAdapter *etta)
@@ -864,22 +833,6 @@ etta_proxy_node_data_changed (ETreeModel *etm,
 	e_table_model_row_changed (E_TABLE_MODEL (etta), row);
 }
 
-static void
-etta_proxy_node_col_changed (ETreeModel *etm,
-                             ETreePath path,
-                             gint col,
-                             ETreeTableAdapter *etta)
-{
-	gint row = get_row (etta, path);
-
-	if (row == -1) {
-		e_table_model_no_change (E_TABLE_MODEL (etta));
-		return;
-	}
-
-	e_table_model_cell_changed (E_TABLE_MODEL (etta), col, row);
-}
-
 static void
 etta_proxy_node_inserted (ETreeModel *etm,
                           ETreePath parent,
@@ -905,14 +858,6 @@ etta_proxy_node_removed (ETreeModel *etm,
 	e_table_model_changed (E_TABLE_MODEL (etta));
 }
 
-static void
-etta_proxy_node_request_collapse (ETreeModel *etm,
-                                  ETreePath node,
-                                  ETreeTableAdapter *etta)
-{
-	e_tree_table_adapter_node_set_expanded (etta, node, FALSE);
-}
-
 static void
 etta_sort_info_changed (ETableSortInfo *sort_info,
                         ETreeTableAdapter *etta)
@@ -970,9 +915,6 @@ e_tree_table_adapter_construct (ETreeTableAdapter *etta,
 	etta->priv->pre_change_id = g_signal_connect (
 		source, "pre_change",
 		G_CALLBACK (etta_proxy_pre_change), etta);
-	etta->priv->no_change_id = g_signal_connect (
-		source, "no_change",
-		G_CALLBACK (etta_proxy_no_change), etta);
 	etta->priv->rebuilt_id = g_signal_connect (
 		source, "rebuilt",
 		G_CALLBACK (etta_proxy_rebuilt), etta);
@@ -982,18 +924,12 @@ e_tree_table_adapter_construct (ETreeTableAdapter *etta,
 	etta->priv->node_data_changed_id = g_signal_connect (
 		source, "node_data_changed",
 		G_CALLBACK (etta_proxy_node_data_changed), etta);
-	etta->priv->node_col_changed_id = g_signal_connect (
-		source, "node_col_changed",
-		G_CALLBACK (etta_proxy_node_col_changed), etta);
 	etta->priv->node_inserted_id = g_signal_connect (
 		source, "node_inserted",
 		G_CALLBACK (etta_proxy_node_inserted), etta);
 	etta->priv->node_removed_id = g_signal_connect (
 		source, "node_removed",
 		G_CALLBACK (etta_proxy_node_removed), etta);
-	etta->priv->node_request_collapse_id = g_signal_connect (
-		source, "node_request_collapse",
-		G_CALLBACK (etta_proxy_node_request_collapse), etta);
 
 	return E_TABLE_MODEL (etta);
 }
