@@ -97,7 +97,6 @@ enum {
 	PROP_DRAW_FOCUS,
 	PROP_ETTA,
 	PROP_UNIFORM_ROW_HEIGHT,
-	PROP_IS_EDITING,
 	PROP_ALWAYS_SEARCH,
 	PROP_HADJUSTMENT,
 	PROP_VADJUSTMENT,
@@ -158,8 +157,6 @@ struct _ETreePrivate {
 
 	gint length_threshold;
 
-	GtkAdjustment *table_canvas_vadjustment;
-
 	/*
 	 * Configuration settings
 	 */
@@ -262,16 +259,6 @@ static void context_destroyed (gpointer data, GObject *ctx);
 G_DEFINE_TYPE_WITH_CODE (ETree, e_tree, GTK_TYPE_TABLE,
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
 
-static void
-tree_item_is_editing_changed_cb (ETableItem *item,
-                                 GParamSpec *param,
-                                 ETree *tree)
-{
-	g_return_if_fail (E_IS_TREE (tree));
-
-	g_object_notify (G_OBJECT (tree), "is-editing");
-}
-
 static void
 et_disconnect_from_etta (ETree *et)
 {
@@ -499,16 +486,10 @@ et_dispose (GObject *object)
 	priv->info_text_resize_id = 0;
 
 	if (priv->table_canvas != NULL) {
-		g_signal_handlers_disconnect_by_data (priv->table_canvas, object);
 		gtk_widget_destroy (GTK_WIDGET (priv->table_canvas));
 		priv->table_canvas = NULL;
 	}
 
-	if (priv->table_canvas_vadjustment) {
-		g_signal_handlers_disconnect_by_data (priv->table_canvas_vadjustment, object);
-		g_clear_object (&priv->table_canvas_vadjustment);
-	}
-
 	/* do not unref it, it was owned by priv->table_canvas */
 	priv->item = NULL;
 
@@ -576,13 +557,11 @@ et_search_search (ETableSearch *search,
 	}
 
 	found = e_tree_model_node_find (
-			et->priv->model, cursor, NULL,
-			E_TREE_FIND_NEXT_FORWARD,
-			search_search_callback, &cb_data);
+		et->priv->model, cursor, NULL,
+		search_search_callback, &cb_data);
 	if (found == NULL)
 		found = e_tree_model_node_find (
 			et->priv->model, NULL, cursor,
-			E_TREE_FIND_NEXT_FORWARD,
 			search_search_callback, &cb_data);
 
 	if (found && found != cursor) {
@@ -1156,9 +1135,6 @@ et_build_item (ETree *et)
 	g_signal_connect (
 		et->priv->item, "start_drag",
 		G_CALLBACK (item_start_drag), et);
-	g_signal_connect (
-		et->priv->item, "notify::is-editing",
-		G_CALLBACK (tree_item_is_editing_changed_cb), et);
 }
 
 static void
@@ -1243,40 +1219,6 @@ table_canvas_focus_event_cb (GtkWidget *widget,
 	return TRUE;
 }
 
-static void
-e_tree_table_canvas_scrolled_cb (GtkAdjustment *vadjustment,
-				 GParamSpec *param,
-				 ETree *tree)
-{
-	g_return_if_fail (E_IS_TREE (tree));
-
-	if (tree->priv->item)
-		e_table_item_cursor_scrolled (E_TABLE_ITEM (tree->priv->item));
-}
-
-static void
-et_setup_table_canvas_vadjustment (ETree *tree)
-{
-	GtkAdjustment *vadjustment = NULL;
-
-	g_return_if_fail (E_IS_TREE (tree));
-
-	if (tree->priv->table_canvas_vadjustment) {
-		g_signal_handlers_disconnect_by_data (tree->priv->table_canvas_vadjustment, tree);
-		g_clear_object (&tree->priv->table_canvas_vadjustment);
-	}
-
-	if (tree->priv->table_canvas)
-		vadjustment = gtk_scrollable_get_vadjustment (
-			GTK_SCROLLABLE (tree->priv->table_canvas));
-
-	if (vadjustment) {
-		tree->priv->table_canvas_vadjustment = g_object_ref (vadjustment);
-		g_signal_connect (vadjustment, "notify::value",
-			G_CALLBACK (e_tree_table_canvas_scrolled_cb), tree);
-	}
-}
-
 static void
 e_tree_setup_table (ETree *e_tree)
 {
@@ -1323,10 +1265,6 @@ e_tree_setup_table (ETree *e_tree)
 		e_tree->priv->table_canvas, "reflow",
 		G_CALLBACK (tree_canvas_reflow), e_tree);
 
-	et_setup_table_canvas_vadjustment (e_tree);
-	g_signal_connect_swapped (e_tree->priv->table_canvas, "notify::vadjustment",
-		G_CALLBACK (et_setup_table_canvas_vadjustment), e_tree);
-
 	widget = GTK_WIDGET (e_tree->priv->table_canvas);
 	style = gtk_widget_get_style (widget);
 
@@ -2057,10 +1995,6 @@ et_get_property (GObject *object,
 		g_value_set_boolean (value, etree->priv->uniform_row_height);
 		break;
 
-	case PROP_IS_EDITING:
-		g_value_set_boolean (value, e_tree_is_editing (etree));
-		break;
-
 	case PROP_ALWAYS_SEARCH:
 		g_value_set_boolean (value, etree->priv->always_search);
 		break;
@@ -3241,12 +3175,10 @@ hover_timeout (gpointer data)
 	path = e_tree_table_adapter_node_at_row (et->priv->etta, row);
 	if (path && e_tree_model_node_is_expandable (et->priv->model, path)) {
 		if (!e_tree_table_adapter_node_is_expanded (et->priv->etta, path)) {
-			if (e_tree_model_has_save_id (et->priv->model) &&
-				e_tree_model_has_get_node_by_id (et->priv->model))
-				et->priv->expanded_list = g_list_prepend (
-					et->priv->expanded_list,
-					e_tree_model_get_save_id (
-					et->priv->model, path));
+			et->priv->expanded_list = g_list_prepend (
+				et->priv->expanded_list,
+				e_tree_model_get_save_id (
+				et->priv->model, path));
 
 			e_tree_table_adapter_node_set_expanded (
 				et->priv->etta, path, TRUE);
@@ -3824,16 +3756,6 @@ e_tree_class_init (ETreeClass *class)
 			FALSE,
 			G_PARAM_READWRITE));
 
-	g_object_class_install_property (
-		object_class,
-		PROP_IS_EDITING,
-		g_param_spec_boolean (
-			"is-editing",
-			"Whether is in an editing mode",
-			"Whether is in an editing mode",
-			FALSE,
-			G_PARAM_READABLE));
-
 	g_object_class_install_property (
 		object_class,
 		PROP_ALWAYS_SEARCH,
@@ -3976,11 +3898,3 @@ e_tree_thaw_state_change (ETree *tree)
 		e_tree_state_change (tree);
 	}
 }
-
-gboolean
-e_tree_is_editing (ETree *tree)
-{
-	g_return_val_if_fail (E_IS_TREE (tree), FALSE);
-
-	return tree->priv->item && e_table_item_is_editing (E_TABLE_ITEM (tree->priv->item));
-}
