@@ -209,12 +209,9 @@
 #include <e-util/e-text-model.h>
 #include <e-util/e-text.h>
 #include <e-util/e-timezone-dialog.h>
-#include <e-util/e-tree-memory-callbacks.h>
-#include <e-util/e-tree-memory.h>
 #include <e-util/e-tree-model-generator.h>
 #include <e-util/e-tree-model.h>
 #include <e-util/e-tree-selection-model.h>
-#include <e-util/e-tree-sorted.h>
 #include <e-util/e-tree-table-adapter.h>
 #include <e-util/e-tree.h>
 #include <e-util/e-unicode.h>
