@@ -31,6 +31,7 @@
 #include "e-mail-formatter-extension.h"
 #include "e-mail-formatter-utils.h"
 #include "e-mail-inline-filter.h"
+#include "e-mail-part-headers.h"
 
 typedef EMailFormatterExtension EMailFormatterHeaders;
 typedef EMailFormatterExtensionClass EMailFormatterHeadersClass;
@@ -50,10 +51,11 @@ static const gchar *formatter_mime_types[] = {
 static void
 format_short_headers (EMailFormatter *formatter,
                       GString *buffer,
-                      CamelMedium *part,
+                      EMailPart *part,
                       guint32 flags,
                       GCancellable *cancellable)
 {
+	CamelMimePart *mime_part;
 	GtkTextDirection direction;
 	const gchar *charset;
 	CamelContentType *ct;
@@ -67,9 +69,10 @@ format_short_headers (EMailFormatter *formatter,
 	if (g_cancellable_is_cancelled (cancellable))
 		return;
 
+	mime_part = e_mail_part_ref_mime_part (part);
 	direction = gtk_widget_get_default_direction ();
 
-	ct = camel_mime_part_get_content_type ((CamelMimePart *) part);
+	ct = camel_mime_part_get_content_type (mime_part);
 	charset = camel_content_type_param (ct, "charset");
 	charset = camel_iconv_charset_name (charset);
 	hdr_charset = e_mail_formatter_dup_charset (formatter);
@@ -85,7 +88,7 @@ format_short_headers (EMailFormatter *formatter,
 		"id=\"__evo-short-headers\" style=\"display: %s\">",
 		flags & E_MAIL_FORMATTER_HEADER_FLAG_COLLAPSED ? "table" : "none");
 
-	header = ((CamelMimePart *) part)->headers;
+	header = mime_part->headers;
 	while (header) {
 		if (!g_ascii_strcasecmp (header->name, "From")) {
 			GString *tmp;
@@ -141,38 +144,40 @@ format_short_headers (EMailFormatter *formatter,
 
 	g_string_free (from, TRUE);
 	g_free (evolution_imagesdir);
+
+	g_object_unref (mime_part);
 }
 
 static void
-write_contact_picture (CamelMimePart *part,
+write_contact_picture (CamelMimePart *mime_part,
                        gint size,
                        GString *buffer)
 {
 	gchar *b64, *content_type;
 	CamelDataWrapper *dw;
 	CamelContentType *ct;
-	GByteArray *ba;
+	GByteArray *ba = NULL;
 
-	ba = NULL;
-	dw = camel_medium_get_content (CAMEL_MEDIUM (part));
-	if (dw) {
+	dw = camel_medium_get_content (CAMEL_MEDIUM (mime_part));
+	if (dw != NULL)
 		ba = camel_data_wrapper_get_byte_array (dw);
-	}
 
-	if (!ba || ba->len == 0) {
+	if (ba == NULL || ba->len == 0) {
+		const gchar *filename;
 
-		if (camel_mime_part_get_filename (part)) {
+		filename = camel_mime_part_get_filename (mime_part);
 
+		if (filename != NULL) {
 			if (size >= 0) {
 				g_string_append_printf (
 					buffer,
 					"<img width=\"%d\" src=\"evo-file://%s\" />",
-					size, camel_mime_part_get_filename (part));
+					size, filename);
 			} else {
 				g_string_append_printf (
 					buffer,
 					"<img src=\"evo-file://%s\" />",
-					camel_mime_part_get_filename (part));
+					filename);
 			}
 		}
 
@@ -180,7 +185,7 @@ write_contact_picture (CamelMimePart *part,
 	}
 
 	b64 = g_base64_encode (ba->data, ba->len);
-	ct = camel_mime_part_get_content_type (part);
+	ct = camel_mime_part_get_content_type (mime_part);
 	content_type = camel_content_type_simple (ct);
 
 	if (size >= 0) {
@@ -199,61 +204,35 @@ write_contact_picture (CamelMimePart *part,
 	g_free (content_type);
 }
 
-static CamelMimePart *
-load_picture_from_file (const gchar *mime_type,
-                         const gchar *filename,
-                         GCancellable *cancellable)
-{
-	CamelMimePart *part;
-	CamelStream *stream;
-	CamelDataWrapper *dw;
-	gchar *basename;
-
-	stream = camel_stream_fs_new_with_name (filename, O_RDONLY, 0, NULL);
-	if (stream == NULL)
-		return NULL;
-
-	dw = camel_data_wrapper_new ();
-	camel_data_wrapper_construct_from_stream_sync (
-		dw, stream, cancellable, NULL);
-	g_object_unref (stream);
-	if (mime_type)
-		camel_data_wrapper_set_mime_type (dw, mime_type);
-	part = camel_mime_part_new ();
-	camel_medium_set_content ((CamelMedium *) part, dw);
-	g_object_unref (dw);
-	basename = g_path_get_basename (filename);
-	camel_mime_part_set_filename (part, basename);
-	g_free (basename);
-
-	return part;
-}
-
 static void
 format_full_headers (EMailFormatter *formatter,
                      GString *buffer,
-                     CamelMedium *part,
+                     EMailPart *part,
                      guint32 mode,
                      guint32 flags,
                      GCancellable *cancellable)
 {
+	CamelMimePart *mime_part;
 	const gchar *charset;
 	CamelContentType *ct;
 	struct _camel_header_raw *header;
-	gboolean have_icon = FALSE;
 	const gchar *photo_name = NULL;
-	gboolean face_decoded  = FALSE, contact_has_photo = FALSE;
 	guchar *face_header_value = NULL;
 	gsize face_header_len = 0;
 	gchar *header_sender = NULL, *header_from = NULL, *name;
 	gboolean mail_from_delegate = FALSE;
+	gboolean show_sender_photo;
 	gchar *hdr_charset;
 	gchar *evolution_imagesdir;
 	const gchar *direction;
 
+	g_return_if_fail (E_IS_MAIL_PART_HEADERS (part));
+
 	if (g_cancellable_is_cancelled (cancellable))
 		return;
 
+	mime_part = e_mail_part_ref_mime_part (part);
+
 	switch (gtk_widget_get_default_direction ()) {
 		case GTK_TEXT_DIR_RTL:
 			direction = "rtl";
@@ -266,7 +245,7 @@ format_full_headers (EMailFormatter *formatter,
 			break;
 	}
 
-	ct = camel_mime_part_get_content_type ((CamelMimePart *) part);
+	ct = camel_mime_part_get_content_type (mime_part);
 	charset = camel_content_type_param (ct, "charset");
 	charset = camel_iconv_charset_name (charset);
 	hdr_charset = e_mail_formatter_dup_charset (formatter);
@@ -284,7 +263,7 @@ format_full_headers (EMailFormatter *formatter,
 		flags & E_MAIL_FORMATTER_HEADER_FLAG_COLLAPSED ? "none" : "table",
 		direction);
 
-	header = ((CamelMimePart *) part)->headers;
+	header = mime_part->headers;
 	while (header != NULL) {
 		if (!g_ascii_strcasecmp (header->name, "Sender")) {
 			struct _camel_header_address *addrs;
@@ -370,66 +349,66 @@ format_full_headers (EMailFormatter *formatter,
 
 	/* dump selected headers */
 	if (mode & E_MAIL_FORMATTER_MODE_ALL_HEADERS) {
-		header = ((CamelMimePart *) part)->headers;
+		header = mime_part->headers;
 		while (header != NULL) {
 			e_mail_formatter_format_header (
-				formatter, buffer, part, header,
+				formatter, buffer,
+				header->name,
+				header->value,
 				E_MAIL_FORMATTER_HEADER_FLAG_NOCOLUMNS, charset);
 			header = header->next;
 		}
 	} else {
-		GQueue *headers_queue;
-		GList *link;
-		gint mailer_shown = FALSE;
+		CamelMedium *medium;
+		gchar **default_headers;
+		guint ii, length = 0;
 
-		headers_queue = e_mail_formatter_dup_headers (formatter);
-		link = g_queue_peek_head_link (headers_queue);
+		medium = CAMEL_MEDIUM (mime_part);
 
-		while (link != NULL) {
-			EMailFormatterHeader *h = link->data;
-			gint mailer, face;
+		default_headers =
+			e_mail_part_headers_dup_default_headers (
+			E_MAIL_PART_HEADERS (part));
+		if (default_headers != NULL)
+			length = g_strv_length (default_headers);
 
-			header = ((CamelMimePart *) part)->headers;
-			mailer = !g_ascii_strcasecmp (h->name, "X-Evolution-Mailer");
-			face = !g_ascii_strcasecmp (h->name, "Face");
+		for (ii = 0; ii < length; ii++) {
+			const gchar *header_name;
+			const gchar *header_value = NULL;
 
-			while (header != NULL) {
-				if (e_mail_formatter_get_show_sender_photo (formatter) &&
-					!photo_name && !g_ascii_strcasecmp (header->name, "From"))
-					photo_name = header->value;
+			header_name = default_headers[ii];
 
-				if (!mailer_shown && mailer && (
-				    !g_ascii_strcasecmp (header->name, "X-Mailer") ||
-				    !g_ascii_strcasecmp (header->name, "User-Agent") ||
-				    !g_ascii_strcasecmp (header->name, "X-Newsreader") ||
-				    !g_ascii_strcasecmp (header->name, "X-MimeOLE"))) {
-					struct _camel_header_raw xmailer, *use_header = NULL;
+			/* X-Evolution-Mailer is a pseudo-header and
+			 * requires special treatment to extract the
+			 * real header value. */
+			if (g_ascii_strcasecmp (header_name, "X-Evolution-Mailer") == 0) {
+				/* Check for "X-MimeOLE" last,
+				 * as it's the least preferred. */
+				if (header_value == NULL)
+					header_value = camel_medium_get_header (
+						medium, "X-Mailer");
+				if (header_value == NULL)
+					header_value = camel_medium_get_header (
+						medium, "User-Agent");
+				if (header_value == NULL)
+					header_value = camel_medium_get_header (
+						medium, "X-Newsreader");
+				if (header_value == NULL)
+					header_value = camel_medium_get_header (
+						medium, "X-MimeOLE");
+			} else {
+				header_value = camel_medium_get_header (
+					medium, header_name);
+			}
 
-					if (!g_ascii_strcasecmp (header->name, "X-MimeOLE")) {
-						for (use_header = header->next; use_header; use_header = use_header->next) {
-							if (!g_ascii_strcasecmp (use_header->name, "X-Mailer") ||
-							    !g_ascii_strcasecmp (use_header->name, "User-Agent") ||
-							    !g_ascii_strcasecmp (use_header->name, "X-Newsreader")) {
-								/* even we have X-MimeOLE, then use rather the standard one, when available */
-								break;
-							}
-						}
-					}
+			if (header_value == NULL)
+				continue;
 
-					if (!use_header)
-						use_header = header;
+			if (g_ascii_strcasecmp (header_name, "From") == 0)
+				photo_name = header_value;
 
-					xmailer.name = (gchar *) "X-Evolution-Mailer";
-					xmailer.value = use_header->value;
-					mailer_shown = TRUE;
-
-					e_mail_formatter_format_header (
-						formatter, buffer, part,
-						&xmailer, h->flags, charset);
-					if (strstr (use_header->value, "Evolution"))
-						have_icon = TRUE;
-				} else if (!face_decoded && face && !g_ascii_strcasecmp (header->name, "Face")) {
-					gchar *cp = header->value;
+			if (g_ascii_strcasecmp (header_name, "Face") == 0) {
+				if (face_header_value == NULL) {
+					const gchar *cp = header_value;
 
 					/* Skip over spaces */
 					while (*cp == ' ')
@@ -441,26 +420,27 @@ format_full_headers (EMailFormatter *formatter,
 						face_header_value,
 						face_header_len + 1);
 					face_header_value[face_header_len] = 0;
-					face_decoded = TRUE;
-				/* Showing an encoded "Face" header makes little sense */
-				} else if (!g_ascii_strcasecmp (header->name, h->name) && !face) {
-					e_mail_formatter_format_header (
-						formatter, buffer, part,
-						header, h->flags, charset);
 				}
-
-				header = header->next;
+				continue;
 			}
 
-			link = g_list_next (link);
+			e_mail_formatter_format_header (
+				formatter, buffer,
+				header_name,
+				header_value,
+				0, charset);
 		}
 
-		g_queue_free_full (headers_queue, (GDestroyNotify) e_mail_formatter_header_free);
+		g_strfreev (default_headers);
 	}
 
 	g_string_append (buffer, "</table></td>");
 
-	if (photo_name) {
+	show_sender_photo =
+		e_mail_formatter_get_show_sender_photo (formatter);
+
+	/* Prefer contact photos over archaic "Face" headers. */
+	if (show_sender_photo && photo_name != NULL) {
 		gchar *name;
 
 		name = g_uri_escape_string (photo_name, NULL, FALSE);
@@ -475,53 +455,29 @@ format_full_headers (EMailFormatter *formatter,
 		g_string_append (buffer, "</td>");
 
 		g_free (name);
-	}
 
-	if (!contact_has_photo && face_decoded) {
-		CamelMimePart *part;
+	} else if (face_header_value != NULL) {
+		CamelMimePart *image_part;
 
-		part = camel_mime_part_new ();
+		image_part = camel_mime_part_new ();
 		camel_mime_part_set_content (
-			(CamelMimePart *) part,
+			image_part,
 			(const gchar *) face_header_value,
 			face_header_len, "image/png");
 
 		g_string_append (
 			buffer,
 			"<td align=\"right\" valign=\"top\">");
-		write_contact_picture (part, 48, buffer);
+		write_contact_picture (image_part, 48, buffer);
 		g_string_append (buffer, "</td>");
 
-		g_object_unref (part);
+		g_object_unref (image_part);
 		g_free (face_header_value);
 	}
 
-	if (have_icon) {
-		GtkIconInfo *icon_info;
-		CamelMimePart *iconpart = NULL;
-
-		icon_info = gtk_icon_theme_lookup_icon (
-			gtk_icon_theme_get_default (),
-			"evolution", 16, GTK_ICON_LOOKUP_NO_SVG);
-		if (icon_info != NULL) {
-			iconpart = load_picture_from_file (
-				"image/png",
-				gtk_icon_info_get_filename (icon_info),
-				cancellable);
-			gtk_icon_info_free (icon_info);
-		}
-		if (iconpart != NULL) {
-			g_string_append (
-				buffer,
-				"<td align=\"right\" valign=\"top\">");
-			write_contact_picture (iconpart, 16, buffer);
-			g_string_append (buffer, "</td>");
-
-			g_object_unref (iconpart);
-		}
-	}
-
 	g_string_append (buffer, "</tr></table>");
+
+	g_object_unref (mime_part);
 }
 
 static gboolean
@@ -600,14 +556,14 @@ emfe_headers_format (EMailFormatterExtension *extension,
 
 	if (is_collapsable)
 		format_short_headers (
-			formatter, buffer,
-			CAMEL_MEDIUM (mime_part),
+			formatter,
+			buffer, part,
 			context->flags,
 			cancellable);
 
 	format_full_headers (
-		formatter, buffer,
-		CAMEL_MEDIUM (mime_part),
+		formatter,
+		buffer, part,
 		context->mode,
 		context->flags,
 		cancellable);
