@@ -56,7 +56,6 @@ struct _EMailFormatterQuoteContext {
 
 struct _EMailFormatterQuote {
 	EMailFormatter parent;
-
 	EMailFormatterQuotePrivate *priv;
 };
 
@@ -85,16 +84,8 @@ G_BEGIN_DECLS
  * This is an abstract base type for formatter extensions which are
  * intended only for use by #EMailFormatterQuote.
  **/
-typedef struct _EMailFormatterQuoteExtension EMailFormatterQuoteExtension;
-typedef struct _EMailFormatterQuoteExtensionClass EMailFormatterQuoteExtensionClass;
-
-struct _EMailFormatterQuoteExtension {
-	EMailFormatterExtension parent;
-};
-
-struct _EMailFormatterQuoteExtensionClass {
-	EMailFormatterExtensionClass parent_class;
-};
+typedef EMailFormatterExtension EMailFormatterQuoteExtension;
+typedef EMailFormatterExtensionClass EMailFormatterQuoteExtensionClass;
 
 GType		e_mail_formatter_quote_extension_get_type
 						(void) G_GNUC_CONST;
