@@ -28,13 +28,15 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
-#include "shell/e-shell.h"
-#include "shell/e-shell-utils.h"
+#include <shell/e-shell.h>
+#include <shell/e-shell-utils.h>
 
-#include "mail/e-mail-reader.h"
-#include "mail/e-mail-reader-utils.h"
-#include "mail/em-folder-tree-model.h"
-#include "mail/message-list.h"
+#include <em-format/e-mail-formatter-enumtypes.h>
+
+#include "e-mail-reader.h"
+#include "e-mail-reader-utils.h"
+#include "em-folder-tree-model.h"
+#include "message-list.h"
 
 #define E_MAIL_BROWSER_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
@@ -48,7 +50,8 @@ struct _EMailBrowserPrivate {
 	GtkUIManager *ui_manager;
 	EFocusTracker *focus_tracker;
 
-	EMailFormatterMode mode;
+	EMailFormatterMode display_mode;
+	EAutomaticActionPolicy close_on_reply_policy;
 
 	GtkWidget *main_menu;
 	GtkWidget *main_toolbar;
@@ -56,19 +59,23 @@ struct _EMailBrowserPrivate {
 	GtkWidget *preview_pane;
 	GtkWidget *statusbar;
 
+	EAlert *close_on_reply_alert;
+	gulong close_on_reply_response_handler_id;
+
 	guint show_deleted : 1;
 };
 
 enum {
 	PROP_0,
 	PROP_BACKEND,
+	PROP_CLOSE_ON_REPLY_POLICY,
+	PROP_DISPLAY_MODE,
 	PROP_FOCUS_TRACKER,
 	PROP_FORWARD_STYLE,
 	PROP_GROUP_BY_THREADS,
-	PROP_SHOW_DELETED,
 	PROP_REPLY_STYLE,
-	PROP_UI_MANAGER,
-	PROP_DISPLAY_MODE,
+	PROP_SHOW_DELETED,
+	PROP_UI_MANAGER
 };
 
 /* This is too trivial to put in a file.
@@ -255,8 +262,6 @@ mail_browser_message_selected_cb (EMailBrowser *browser,
 	CamelMessageInfo *info;
 	CamelFolder *folder;
 	EMailReader *reader;
-	EMailDisplay *display;
-	const gchar *title;
 	guint32 state;
 
 	reader = E_MAIL_READER (browser);
@@ -266,24 +271,30 @@ mail_browser_message_selected_cb (EMailBrowser *browser,
 	if (uid == NULL)
 		return;
 
-	folder = e_mail_reader_get_folder (reader);
-	display = e_mail_reader_get_mail_display (reader);
+	folder = e_mail_reader_ref_folder (reader);
 
 	info = camel_folder_get_message_info (folder, uid);
 
-	if (info == NULL)
-		return;
+	if (info != NULL) {
+		EMailDisplay *display;
+		const gchar *title;
 
-	title = camel_message_info_subject (info);
-	if (title == NULL || *title == '\0')
-		title = _("(No Subject)");
+		display = e_mail_reader_get_mail_display (reader);
 
-	gtk_window_set_title (GTK_WINDOW (browser), title);
-	gtk_widget_grab_focus (GTK_WIDGET (display));
+		title = camel_message_info_subject (info);
+		if (title == NULL || *title == '\0')
+			title = _("(No Subject)");
 
-	camel_message_info_set_flags (
-		info, CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
-	camel_folder_free_message_info (folder, info);
+		gtk_window_set_title (GTK_WINDOW (browser), title);
+		gtk_widget_grab_focus (GTK_WIDGET (display));
+
+		camel_message_info_set_flags (
+			info, CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
+
+		camel_folder_free_message_info (folder, info);
+	}
+
+	g_clear_object (&folder);
 }
 
 static gboolean
@@ -300,7 +311,7 @@ mail_browser_message_list_built_cb (EMailBrowser *browser,
 	g_return_if_fail (E_IS_MAIL_BROWSER (browser));
 	g_return_if_fail (IS_MESSAGE_LIST (message_list));
 
-	if (!message_list_count (message_list))
+	if (message_list_count (message_list) == 0)
 		/* Prioritize ahead of GTK+ redraws. */
 		g_idle_add_full (
 			G_PRIORITY_HIGH_IDLE,
@@ -354,6 +365,30 @@ mail_browser_status_message_cb (EMailBrowser *browser,
 		gtk_statusbar_push (statusbar, context_id, status_message);
 }
 
+static void
+mail_browser_close_on_reply_response_cb (EAlert *alert,
+                                         gint response_id,
+                                         EMailBrowser *browser)
+{
+	/* Disconnect the signal handler, but leave the EAlert
+	 * itself in place so we know it's already been presented. */
+	g_signal_handler_disconnect (
+		browser->priv->close_on_reply_alert,
+		browser->priv->close_on_reply_response_handler_id);
+	browser->priv->close_on_reply_response_handler_id = 0;
+
+	if (response_id == GTK_RESPONSE_YES) {
+		e_mail_browser_set_close_on_reply_policy (
+			browser, E_AUTOMATIC_ACTION_POLICY_ALWAYS);
+		e_mail_browser_close (browser);
+	}
+
+	if (response_id == GTK_RESPONSE_NO) {
+		e_mail_browser_set_close_on_reply_policy (
+			browser, E_AUTOMATIC_ACTION_POLICY_NEVER);
+	}
+}
+
 static void
 mail_browser_set_backend (EMailBrowser *browser,
                           EMailBackend *backend)
@@ -364,6 +399,13 @@ mail_browser_set_backend (EMailBrowser *browser,
 	browser->priv->backend = g_object_ref (backend);
 }
 
+static void
+mail_browser_set_display_mode (EMailBrowser *browser,
+                               EMailFormatterMode display_mode)
+{
+	browser->priv->display_mode = display_mode;
+}
+
 static void
 mail_browser_set_property (GObject *object,
                            guint property_id,
@@ -377,6 +419,18 @@ mail_browser_set_property (GObject *object,
 				g_value_get_object (value));
 			return;
 
+		case PROP_CLOSE_ON_REPLY_POLICY:
+			e_mail_browser_set_close_on_reply_policy (
+				E_MAIL_BROWSER (object),
+				g_value_get_enum (value));
+			return;
+
+		case PROP_DISPLAY_MODE:
+			mail_browser_set_display_mode (
+				E_MAIL_BROWSER (object),
+				g_value_get_enum (value));
+			return;
+
 		case PROP_FORWARD_STYLE:
 			e_mail_reader_set_forward_style (
 				E_MAIL_READER (object),
@@ -400,11 +454,6 @@ mail_browser_set_property (GObject *object,
 				E_MAIL_BROWSER (object),
 				g_value_get_boolean (value));
 			return;
-
-		case PROP_DISPLAY_MODE:
-			E_MAIL_BROWSER (object)->priv->mode =
-				g_value_get_int (value);
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -419,50 +468,66 @@ mail_browser_get_property (GObject *object,
 	switch (property_id) {
 		case PROP_BACKEND:
 			g_value_set_object (
-				value, e_mail_reader_get_backend (
+				value,
+				e_mail_reader_get_backend (
 				E_MAIL_READER (object)));
 			return;
 
+		case PROP_CLOSE_ON_REPLY_POLICY:
+			g_value_set_enum (
+				value,
+				e_mail_browser_get_close_on_reply_policy (
+				E_MAIL_BROWSER (object)));
+			return;
+
+		case PROP_DISPLAY_MODE:
+			g_value_set_enum (
+				value,
+				e_mail_browser_get_display_mode (
+				E_MAIL_BROWSER (object)));
+			return;
+
 		case PROP_FOCUS_TRACKER:
 			g_value_set_object (
-				value, e_mail_browser_get_focus_tracker (
+				value,
+				e_mail_browser_get_focus_tracker (
 				E_MAIL_BROWSER (object)));
 			return;
 
 		case PROP_FORWARD_STYLE:
 			g_value_set_enum (
-				value, e_mail_reader_get_forward_style (
+				value,
+				e_mail_reader_get_forward_style (
 				E_MAIL_READER (object)));
 			return;
 
 		case PROP_GROUP_BY_THREADS:
 			g_value_set_boolean (
-				value, e_mail_reader_get_group_by_threads (
+				value,
+				e_mail_reader_get_group_by_threads (
 				E_MAIL_READER (object)));
 			return;
 
 		case PROP_REPLY_STYLE:
 			g_value_set_enum (
-				value, e_mail_reader_get_reply_style (
+				value,
+				e_mail_reader_get_reply_style (
 				E_MAIL_READER (object)));
 			return;
 
 		case PROP_SHOW_DELETED:
 			g_value_set_boolean (
-				value, e_mail_browser_get_show_deleted (
+				value,
+				e_mail_browser_get_show_deleted (
 				E_MAIL_BROWSER (object)));
 			return;
 
 		case PROP_UI_MANAGER:
 			g_value_set_object (
-				value, e_mail_browser_get_ui_manager (
+				value,
+				e_mail_browser_get_ui_manager (
 				E_MAIL_BROWSER (object)));
 			return;
-
-		case PROP_DISPLAY_MODE:
-			g_value_set_int (
-				value, E_MAIL_BROWSER (object)->priv->mode);
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -475,30 +540,21 @@ mail_browser_dispose (GObject *object)
 
 	priv = E_MAIL_BROWSER_GET_PRIVATE (object);
 
-	if (priv->backend != NULL) {
-		g_object_unref (priv->backend);
-		priv->backend = NULL;
+	if (priv->close_on_reply_response_handler_id > 0) {
+		g_signal_handler_disconnect (
+			priv->close_on_reply_alert,
+			priv->close_on_reply_response_handler_id);
+		priv->close_on_reply_response_handler_id = 0;
 	}
 
-	if (priv->ui_manager != NULL) {
-		g_object_unref (priv->ui_manager);
-		priv->ui_manager = NULL;
-	}
-
-	if (priv->focus_tracker != NULL) {
-		g_object_unref (priv->focus_tracker);
-		priv->focus_tracker = NULL;
-	}
-
-	if (priv->main_menu != NULL) {
-		g_object_unref (priv->main_menu);
-		priv->main_menu = NULL;
-	}
-
-	if (priv->main_toolbar != NULL) {
-		g_object_unref (priv->main_toolbar);
-		priv->main_toolbar = NULL;
-	}
+	g_clear_object (&priv->backend);
+	g_clear_object (&priv->ui_manager);
+	g_clear_object (&priv->focus_tracker);
+	g_clear_object (&priv->main_menu);
+	g_clear_object (&priv->main_toolbar);
+	g_clear_object (&priv->preview_pane);
+	g_clear_object (&priv->statusbar);
+	g_clear_object (&priv->close_on_reply_alert);
 
 	if (priv->message_list != NULL) {
 		/* This will cancel a regen operation. */
@@ -506,16 +562,6 @@ mail_browser_dispose (GObject *object)
 		priv->message_list = NULL;
 	}
 
-	if (priv->preview_pane != NULL) {
-		g_object_unref (priv->preview_pane);
-		priv->preview_pane = NULL;
-	}
-
-	if (priv->statusbar != NULL) {
-		g_object_unref (priv->statusbar);
-		priv->statusbar = NULL;
-	}
-
 	/* Chain up to parent's dispose() method. */
 	G_OBJECT_CLASS (e_mail_browser_parent_class)->dispose (object);
 }
@@ -527,16 +573,15 @@ mail_browser_constructed (GObject *object)
 	EMailReader *reader;
 	EMailBackend *backend;
 	EMailSession *session;
-	EMailDisplay *display;
 	EShellBackend *shell_backend;
 	EShell *shell;
 	EFocusTracker *focus_tracker;
-	GSettings *settings;
 	GtkAccelGroup *accel_group;
 	GtkActionGroup *action_group;
 	GtkAction *action;
 	GtkUIManager *ui_manager;
 	GtkWidget *container;
+	GtkWidget *display;
 	GtkWidget *widget;
 	const gchar *domain;
 	const gchar *id;
@@ -575,9 +620,11 @@ mail_browser_constructed (GObject *object)
 		browser->priv->message_list, "message-list-built",
 		G_CALLBACK (mail_browser_message_list_built_cb), object);
 
-	display = g_object_new (
-		E_TYPE_MAIL_DISPLAY,
-		"mode", E_MAIL_BROWSER (object)->priv->mode, NULL);
+	display = e_mail_display_new ();
+
+	e_mail_display_set_mode (
+		E_MAIL_DISPLAY (display),
+		browser->priv->display_mode);
 
 	g_signal_connect_swapped (
 		display, "popup-event",
@@ -602,7 +649,6 @@ mail_browser_constructed (GObject *object)
 	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
 
 	/* For easy access.  Takes ownership of the reference. */
-
 	g_object_set_data_full (
 		object, ACTION_GROUP_STANDARD,
 		action_group, (GDestroyNotify) g_object_unref);
@@ -676,17 +722,6 @@ mail_browser_constructed (GObject *object)
 		browser->priv->preview_pane,
 		TRUE, TRUE, 0);
 
-	/* Bind GObject properties to GSettings keys. */
-
-	settings = g_settings_new ("org.gnome.evolution.mail");
-
-	g_settings_bind (
-		settings, "show-deleted",
-		reader, "show-deleted",
-		G_SETTINGS_BIND_DEFAULT);
-
-	g_object_unref (settings);
-
 	id = "org.gnome.evolution.mail.browser";
 	e_plugin_ui_register_manager (ui_manager, id, object);
 	e_plugin_ui_enable_manager (ui_manager, id);
@@ -751,22 +786,23 @@ mail_browser_get_hide_deleted (EMailReader *reader)
 static EMailDisplay *
 mail_browser_get_mail_display (EMailReader *reader)
 {
-	EMailBrowserPrivate *priv;
+	EPreviewPane *preview_pane;
+	EWebView *web_view;
 
-	priv = E_MAIL_BROWSER_GET_PRIVATE (E_MAIL_BROWSER (reader));
+	preview_pane = e_mail_reader_get_preview_pane (reader);
+	web_view = e_preview_pane_get_web_view (preview_pane);
 
-	return E_MAIL_DISPLAY (e_preview_pane_get_web_view (
-					E_PREVIEW_PANE (priv->preview_pane)));
+	return E_MAIL_DISPLAY (web_view);
 }
 
 static GtkWidget *
 mail_browser_get_message_list (EMailReader *reader)
 {
-	EMailBrowser *browser;
+	EMailBrowserPrivate *priv;
 
-	browser = E_MAIL_BROWSER (reader);
+	priv = E_MAIL_BROWSER_GET_PRIVATE (reader);
 
-	return browser->priv->message_list;
+	return priv->message_list;
 }
 
 static GtkMenu *
@@ -786,11 +822,11 @@ mail_browser_get_popup_menu (EMailReader *reader)
 static EPreviewPane *
 mail_browser_get_preview_pane (EMailReader *reader)
 {
-	EMailBrowser *browser;
+	EMailBrowserPrivate *priv;
 
-	browser = E_MAIL_BROWSER (reader);
+	priv = E_MAIL_BROWSER_GET_PRIVATE (reader);
 
-	return E_PREVIEW_PANE (browser->priv->preview_pane);
+	return E_PREVIEW_PANE (priv->preview_pane);
 }
 
 static GtkWindow *
@@ -804,19 +840,23 @@ mail_browser_set_message (EMailReader *reader,
                           const gchar *uid)
 {
 	EMailReaderInterface *interface;
+	EMailBrowser *browser;
 	CamelMessageInfo *info;
 	CamelFolder *folder;
 
+	browser = E_MAIL_BROWSER (reader);
+
 	/* Chain up to parent's set_message() method. */
 	interface = g_type_default_interface_peek (E_TYPE_MAIL_READER);
 	interface->set_message (reader, uid);
 
 	if (uid == NULL) {
-		e_mail_browser_close (E_MAIL_BROWSER (reader));
+		e_mail_browser_close (browser);
 		return;
 	}
 
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
+
 	info = camel_folder_get_message_info (folder, uid);
 
 	if (info != NULL) {
@@ -825,6 +865,40 @@ mail_browser_set_message (EMailReader *reader,
 			camel_message_info_subject (info));
 		camel_folder_free_message_info (folder, info);
 	}
+
+	g_clear_object (&folder);
+}
+
+static void
+mail_browser_composer_created (EMailReader *reader,
+                               EMsgComposer *composer,
+                               CamelMimeMessage *message)
+{
+	EMailBrowser *browser;
+	EAutomaticActionPolicy policy;
+
+	/* Do not prompt if there is no source message.  It means
+	 * the user wants to start a brand new message, presumably
+	 * unrelated to the message shown in the browser window. */
+	if (message == NULL)
+		return;
+
+	browser = E_MAIL_BROWSER (reader);
+	policy = e_mail_browser_get_close_on_reply_policy (browser);
+
+	switch (policy) {
+		case E_AUTOMATIC_ACTION_POLICY_ALWAYS:
+			e_mail_browser_close (browser);
+			break;
+
+		case E_AUTOMATIC_ACTION_POLICY_NEVER:
+			/* do nothing */
+			break;
+
+		case E_AUTOMATIC_ACTION_POLICY_ASK:
+			e_mail_browser_ask_close_on_reply (browser);
+			break;
+	}
 }
 
 static void
@@ -853,7 +927,36 @@ e_mail_browser_class_init (EMailBrowserClass *class)
 			"The mail backend",
 			E_TYPE_MAIL_BACKEND,
 			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT_ONLY));
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_CLOSE_ON_REPLY_POLICY,
+		g_param_spec_enum (
+			"close-on-reply-policy",
+			"Close on Reply Policy",
+			"Policy for automatically closing the message "
+			"browser window when forwarding or replying to "
+			"the displayed message",
+			E_TYPE_AUTOMATIC_ACTION_POLICY,
+			E_AUTOMATIC_ACTION_POLICY_ASK,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_DISPLAY_MODE,
+		g_param_spec_enum (
+			"display-mode",
+			"Display Mode",
+			NULL,
+			E_TYPE_MAIL_FORMATTER_MODE,
+			E_MAIL_FORMATTER_MODE_NORMAL,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (
 		object_class,
@@ -863,7 +966,8 @@ e_mail_browser_class_init (EMailBrowserClass *class)
 			"Focus Tracker",
 			NULL,
 			E_TYPE_FOCUS_TRACKER,
-			G_PARAM_READABLE));
+			G_PARAM_READABLE |
+			G_PARAM_STATIC_STRINGS));
 
 	/* Inherited from EMailReader */
 	g_object_class_override_property (
@@ -891,19 +995,19 @@ e_mail_browser_class_init (EMailBrowserClass *class)
 			"Show Deleted",
 			"Show deleted messages",
 			FALSE,
-			G_PARAM_READWRITE));
+			G_PARAM_READWRITE |
+			G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (
 		object_class,
-		PROP_DISPLAY_MODE,
-		g_param_spec_int (
-			"display-mode",
-			"Display Mode",
+		PROP_UI_MANAGER,
+		g_param_spec_object (
+			"ui-manager",
+			"UI Manager",
 			NULL,
-			0,
-			G_MAXINT,
-			E_MAIL_FORMATTER_MODE_NORMAL,
-			G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+			GTK_TYPE_UI_MANAGER,
+			G_PARAM_READABLE |
+			G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -918,6 +1022,7 @@ e_mail_browser_reader_init (EMailReaderInterface *interface)
 	interface->get_preview_pane = mail_browser_get_preview_pane;
 	interface->get_window = mail_browser_get_window;
 	interface->set_message = mail_browser_set_message;
+	interface->composer_created = mail_browser_composer_created;
 }
 
 static void
@@ -938,19 +1043,15 @@ GtkWidget *
 e_mail_browser_new (EMailBackend *backend,
                     CamelFolder *folder,
                     const gchar *msg_uid,
-                    EMailFormatterMode mode)
+                    EMailFormatterMode display_mode)
 {
-	GtkWidget *widget;
-
 	g_return_val_if_fail (E_IS_MAIL_BACKEND (backend), NULL);
 
-	widget= g_object_new (
+	return g_object_new (
 		E_TYPE_MAIL_BROWSER,
 		"backend", backend,
-		"display-mode", mode,
+		"display-mode", display_mode,
 		NULL);
-
-	return widget;
 }
 
 void
@@ -961,6 +1062,79 @@ e_mail_browser_close (EMailBrowser *browser)
 	gtk_widget_destroy (GTK_WIDGET (browser));
 }
 
+void
+e_mail_browser_ask_close_on_reply (EMailBrowser *browser)
+{
+	EAlertSink *alert_sink;
+	EAlert *alert;
+	gulong handler_id;
+
+	g_return_if_fail (E_IS_MAIL_BROWSER (browser));
+
+	/* Do nothing if the question has already been presented, even if
+	 * the user dismissed it without answering.  We only present the
+	 * question once per browser window, lest it become annoying. */
+	if (browser->priv->close_on_reply_alert != NULL)
+		return;
+
+	alert = e_alert_new ("mail:browser-close-on-reply", NULL);
+
+	handler_id = g_signal_connect (
+		alert, "response",
+		G_CALLBACK (mail_browser_close_on_reply_response_cb),
+		browser);
+
+	browser->priv->close_on_reply_alert = g_object_ref (alert);
+	browser->priv->close_on_reply_response_handler_id = handler_id;
+
+	alert_sink = e_mail_reader_get_alert_sink (E_MAIL_READER (browser));
+	e_alert_sink_submit_alert (alert_sink, alert);
+
+	g_object_unref (alert);
+}
+
+EAutomaticActionPolicy
+e_mail_browser_get_close_on_reply_policy (EMailBrowser *browser)
+{
+	g_return_val_if_fail (
+		E_IS_MAIL_BROWSER (browser),
+		E_AUTOMATIC_ACTION_POLICY_ASK);
+
+	return browser->priv->close_on_reply_policy;
+}
+
+void
+e_mail_browser_set_close_on_reply_policy (EMailBrowser *browser,
+                                          EAutomaticActionPolicy policy)
+{
+	g_return_if_fail (E_IS_MAIL_BROWSER (browser));
+
+	if (policy == browser->priv->close_on_reply_policy)
+		return;
+
+	browser->priv->close_on_reply_policy = policy;
+
+	g_object_notify (G_OBJECT (browser), "close-on-reply-policy");
+}
+
+EMailFormatterMode
+e_mail_browser_get_display_mode (EMailBrowser *browser)
+{
+	g_return_val_if_fail (
+		E_IS_MAIL_BROWSER (browser),
+		E_MAIL_FORMATTER_MODE_INVALID);
+
+	return browser->priv->display_mode;
+}
+
+EFocusTracker *
+e_mail_browser_get_focus_tracker (EMailBrowser *browser)
+{
+	g_return_val_if_fail (E_IS_MAIL_BROWSER (browser), NULL);
+
+	return browser->priv->focus_tracker;
+}
+
 gboolean
 e_mail_browser_get_show_deleted (EMailBrowser *browser)
 {
@@ -983,14 +1157,6 @@ e_mail_browser_set_show_deleted (EMailBrowser *browser,
 	g_object_notify (G_OBJECT (browser), "show-deleted");
 }
 
-EFocusTracker *
-e_mail_browser_get_focus_tracker (EMailBrowser *browser)
-{
-	g_return_val_if_fail (E_IS_MAIL_BROWSER (browser), NULL);
-
-	return browser->priv->focus_tracker;
-}
-
 GtkUIManager *
 e_mail_browser_get_ui_manager (EMailBrowser *browser)
 {
