@@ -63,7 +63,6 @@ struct _EMailPanedViewPrivate {
 
 	/* Signal handler IDs */
 	guint message_list_built_id;
-	guint enable_show_folder : 1;
 };
 
 enum {
@@ -93,31 +92,34 @@ mail_paned_view_save_boolean (EMailView *view,
                               const gchar *key,
                               gboolean value)
 {
-	EShellView *shell_view;
 	EMailReader *reader;
 	CamelFolder *folder;
-	GKeyFile *key_file;
-	gchar *folder_uri;
-	gchar *group_name;
-
-	shell_view = e_mail_view_get_shell_view (view);
-	key_file = e_shell_view_get_state_key_file (shell_view);
 
 	reader = E_MAIL_READER (view);
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 
-	if (folder == NULL)
-		return;
+	if (folder != NULL) {
+		EShellView *shell_view;
+		GKeyFile *key_file;
+		gchar *folder_uri;
+		gchar *group_name;
 
-	folder_uri = e_mail_folder_uri_from_folder (folder);
-	group_name = g_strdup_printf ("Folder %s", folder_uri);
-	g_key_file_set_boolean (key_file, group_name, key, value);
-	g_free (group_name);
-	g_free (folder_uri);
+		shell_view = e_mail_view_get_shell_view (view);
+		key_file = e_shell_view_get_state_key_file (shell_view);
 
-	g_key_file_set_boolean (key_file, STATE_GROUP_GLOBAL_FOLDER, key, value);
+		folder_uri = e_mail_folder_uri_from_folder (folder);
+		group_name = g_strdup_printf ("Folder %s", folder_uri);
+		g_key_file_set_boolean (key_file, group_name, key, value);
+		g_free (group_name);
+		g_free (folder_uri);
 
-	e_shell_view_set_state_dirty (shell_view);
+		g_key_file_set_boolean (
+			key_file, STATE_GROUP_GLOBAL_FOLDER, key, value);
+
+		e_shell_view_set_state_dirty (shell_view);
+
+		g_object_unref (folder);
+	}
 }
 
 static void
@@ -127,10 +129,13 @@ mail_paned_view_message_list_built_cb (EMailView *view,
 	EMailPanedViewPrivate *priv;
 	EShellView *shell_view;
 	EShellWindow *shell_window;
+	CamelFolder *folder;
 	GKeyFile *key_file;
 
 	priv = E_MAIL_PANED_VIEW_GET_PRIVATE (view);
 
+	folder = message_list_ref_folder (message_list);
+
 	g_signal_handler_disconnect (
 		message_list, priv->message_list_built_id);
 	priv->message_list_built_id = 0;
@@ -143,20 +148,18 @@ mail_paned_view_message_list_built_cb (EMailView *view,
 	if (message_list->cursor_uid != NULL)
 		;  /* do nothing */
 
-	else if (message_list->folder == NULL)
+	else if (folder == NULL)
 		;  /* do nothing */
 
 	else if (e_shell_window_get_safe_mode (shell_window))
 		e_shell_window_set_safe_mode (shell_window, FALSE);
 
 	else {
-		CamelFolder *folder;
 		const gchar *key;
 		gchar *folder_uri;
 		gchar *group_name;
 		gchar *uid;
 
-		folder = message_list->folder;
 		folder_uri = e_mail_folder_uri_from_folder (folder);
 
 		key = STATE_KEY_SELECTED_MESSAGE;
@@ -171,6 +174,8 @@ mail_paned_view_message_list_built_cb (EMailView *view,
 
 		g_free (uid);
 	}
+
+	g_clear_object (&folder);
 }
 
 static void
@@ -185,7 +190,7 @@ mail_paned_view_message_selected_cb (EMailView *view,
 	gchar *folder_uri;
 	gchar *group_name;
 
-	folder = message_list->folder;
+	folder = message_list_ref_folder (message_list);
 
 	/* This also gets triggered when selecting a store name on
 	 * the sidebar such as "On This Computer", in which case
@@ -209,6 +214,8 @@ mail_paned_view_message_selected_cb (EMailView *view,
 
 	g_free (group_name);
 	g_free (folder_uri);
+
+	g_object_unref (folder);
 }
 
 static void
@@ -612,16 +619,6 @@ mail_paned_view_reader_open_selected_mail (EMailReader *reader)
 	return class->open_selected_mail (paned_view);
 }
 
-static gboolean
-mail_paned_view_enable_show_folder (EMailReader *reader)
-{
-	EMailPanedView *paned_view;
-
-	paned_view = E_MAIL_PANED_VIEW (reader);
-
-	return paned_view->priv->enable_show_folder;
-}
-
 static void
 mail_paned_view_constructed (GObject *object)
 {
@@ -834,7 +831,7 @@ mail_paned_view_update_view_instance (EMailView *view)
 	registry = e_shell_get_registry (shell);
 
 	reader = E_MAIL_READER (view);
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 
 	/* If no folder is selected, return silently. */
 	if (folder == NULL)
@@ -858,12 +855,18 @@ mail_paned_view_update_view_instance (EMailView *view)
 		settings, "global-view-setting");
 	g_object_unref (settings);
 
-	if (global_view_setting)
+	if (global_view_setting) {
+		if (outgoing_folder) {
+			view_instance = e_shell_view_new_view_instance (
+				shell_view, "global_view_sent_setting");
+		} else {
+			view_instance = e_shell_view_new_view_instance (
+				shell_view, "global_view_setting");
+		}
+	} else {
 		view_instance = e_shell_view_new_view_instance (
-			shell_view, outgoing_folder ?
-			"global_view_sent_setting" : "global_view_setting");
-	else
-		view_instance = e_shell_view_new_view_instance (shell_view, view_id);
+			shell_view, view_id);
+	}
 
 	priv->view_instance = view_instance;
 
@@ -875,29 +878,23 @@ mail_paned_view_update_view_instance (EMailView *view)
 
 	if (show_vertical_view) {
 		gchar *filename;
-		gchar *safe_view_id;
 
 		/* Force the view instance into vertical view. */
 
 		g_free (view_instance->custom_filename);
 		g_free (view_instance->current_view_filename);
 
-		safe_view_id = g_strdup (view_id);
-		e_filename_make_safe (safe_view_id);
-
 		filename = g_strdup_printf (
-			"custom_wide_view-%s.xml", safe_view_id);
+			"custom_wide_view-%s.xml", view_id);
 		view_instance->custom_filename = g_build_filename (
 			view_collection->local_dir, filename, NULL);
 		g_free (filename);
 
 		filename = g_strdup_printf (
-			"current_wide_view-%s.xml", safe_view_id);
+			"current_wide_view-%s.xml", view_id);
 		view_instance->current_view_filename = g_build_filename (
 			view_collection->local_dir, filename, NULL);
 		g_free (filename);
-
-		g_free (safe_view_id);
 	}
 
 	g_free (view_id);
@@ -961,6 +958,8 @@ mail_paned_view_update_view_instance (EMailView *view)
 
 	mail_paned_display_view_cb (
 		view, gal_view_instance_get_current_view (view_instance));
+
+	g_clear_object (&folder);
 }
 
 static void
@@ -1056,14 +1055,12 @@ e_mail_paned_view_reader_init (EMailReaderInterface *interface)
 	interface->get_window = mail_paned_view_get_window;
 	interface->set_folder = mail_paned_view_set_folder;
 	interface->open_selected_mail = mail_paned_view_reader_open_selected_mail;
-	interface->enable_show_folder = mail_paned_view_enable_show_folder;
 }
 
 static void
 e_mail_paned_view_init (EMailPanedView *view)
 {
 	view->priv = E_MAIL_PANED_VIEW_GET_PRIVATE (view);
-	view->priv->enable_show_folder = FALSE;
 
 	g_signal_connect (
 		view, "notify::group-by-threads",
@@ -1101,9 +1098,3 @@ e_mail_paned_view_get_preview (EMailPanedView *view)
 	return GTK_WIDGET (mail_paned_view_get_mail_display (E_MAIL_READER (view)));
 }
 
-void
-e_mail_paned_view_set_enable_show_folder (EMailPanedView *view,
-                                          gboolean set)
-{
-	view->priv->enable_show_folder = set;
-}
