@@ -65,14 +65,18 @@ struct _AsyncContext {
 	EMailPartList *part_list;
 	EMailReader *reader;
 	CamelInternetAddress *address;
+	GPtrArray *uids;
 	gchar *folder_name;
 	gchar *message_uid;
 
 	EMailReplyType reply_type;
 	EMailReplyStyle reply_style;
+	EMailForwardStyle forward_style;
 	GtkPrintOperationAction print_action;
 	const gchar *filter_source;
 	gint filter_type;
+	gboolean replace;
+	gboolean keep_signature;
 };
 
 static void
@@ -85,6 +89,9 @@ async_context_free (AsyncContext *async_context)
 	g_clear_object (&async_context->reader);
 	g_clear_object (&async_context->address);
 
+	if (async_context->uids != NULL)
+		g_ptr_array_unref (async_context->uids);
+
 	g_free (async_context->folder_name);
 	g_free (async_context->message_uid);
 
@@ -119,7 +126,7 @@ e_mail_reader_confirm_delete (EMailReader *reader)
 
 	g_return_val_if_fail (E_IS_MAIL_READER (reader), FALSE);
 
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 	window = e_mail_reader_get_window (reader);
 
 	settings = g_settings_new ("org.gnome.evolution.mail");
@@ -158,6 +165,8 @@ e_mail_reader_confirm_delete (EMailReader *reader)
 	gtk_widget_destroy (dialog);
 
 exit:
+	g_clear_object (&folder);
+
 	return (response == GTK_RESPONSE_OK);
 }
 
@@ -738,29 +747,33 @@ e_mail_reader_mark_selected (EMailReader *reader,
                              guint32 set)
 {
 	CamelFolder *folder;
-	GPtrArray *uids;
-	guint ii;
+	guint ii = 0;
 
 	g_return_val_if_fail (E_IS_MAIL_READER (reader), 0);
 
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 
-	if (folder == NULL)
-		return 0;
+	if (folder != NULL) {
+		GPtrArray *uids;
 
-	camel_folder_freeze (folder);
-	uids = e_mail_reader_get_selected_uids (reader);
+		camel_folder_freeze (folder);
 
-	for (ii = 0; ii < uids->len; ii++)
-		camel_folder_set_message_flags (
-			folder, uids->pdata[ii], mask, set);
+		uids = e_mail_reader_get_selected_uids (reader);
 
-	em_utils_uids_free (uids);
+		for (ii = 0; ii < uids->len; ii++)
+			camel_folder_set_message_flags (
+				folder, uids->pdata[ii], mask, set);
 
-	camel_folder_thaw (folder);
+		g_ptr_array_unref (uids);
+
+		camel_folder_thaw (folder);
+
+		g_object_unref (folder);
+	}
 
 	return ii;
 }
+
 static void
 copy_tree_state (EMailReader *src_reader,
                  EMailReader *des_reader)
@@ -797,7 +810,7 @@ e_mail_reader_open_selected (EMailReader *reader)
 	GtkWindow *window;
 	GPtrArray *views;
 	GPtrArray *uids;
-	guint ii;
+	guint ii = 0;
 
 	g_return_val_if_fail (E_IS_MAIL_READER (reader), 0);
 
@@ -805,20 +818,22 @@ e_mail_reader_open_selected (EMailReader *reader)
 	shell = e_shell_backend_get_shell (E_SHELL_BACKEND (backend));
 	registry = e_shell_get_registry (shell);
 
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 	uids = e_mail_reader_get_selected_uids (reader);
 	window = e_mail_reader_get_window (reader);
 
-	if (!em_utils_ask_open_many (window, uids->len)) {
-		em_utils_uids_free (uids);
-		return 0;
-	}
+	if (!em_utils_ask_open_many (window, uids->len))
+		goto exit;
 
 	if (em_utils_folder_is_drafts (registry, folder) ||
 		em_utils_folder_is_outbox (registry, folder) ||
 		em_utils_folder_is_templates (registry, folder)) {
-		em_utils_edit_messages (reader, folder, uids, TRUE, TRUE);
-		return uids->len;
+
+		e_mail_reader_edit_messages (reader, folder, uids, TRUE, TRUE);
+
+		ii = uids->len;
+
+		goto exit;
 	}
 
 	views = g_ptr_array_new ();
@@ -849,8 +864,9 @@ e_mail_reader_open_selected (EMailReader *reader)
 
 			edits = g_ptr_array_new ();
 			g_ptr_array_add (edits, real_uid);
-			em_utils_edit_messages (
+			e_mail_reader_edit_messages (
 				reader, real_folder, edits, TRUE, TRUE);
+			g_ptr_array_unref (edits);
 		} else {
 			g_free (real_uid);
 			g_ptr_array_add (views, g_strdup (uid));
@@ -887,7 +903,9 @@ e_mail_reader_open_selected (EMailReader *reader)
 	g_ptr_array_foreach (views, (GFunc) g_free, NULL);
 	g_ptr_array_free (views, TRUE);
 
-	em_utils_uids_free (uids);
+exit:
+	g_clear_object (&folder);
+	g_ptr_array_unref (uids);
 
 	return ii;
 }
@@ -1028,14 +1046,12 @@ e_mail_reader_print (EMailReader *reader,
                      GtkPrintOperationAction action)
 {
 	EActivity *activity;
-	CamelFolder *folder;
 	GCancellable *cancellable;
 	MessageList *message_list;
 	AsyncContext *async_context;
 
 	g_return_if_fail (E_IS_MAIL_READER (reader));
 
-	folder = e_mail_reader_get_folder (reader);
 	message_list = MESSAGE_LIST (e_mail_reader_get_message_list (reader));
 
 	activity = e_mail_reader_new_activity (reader);
@@ -1043,7 +1059,7 @@ e_mail_reader_print (EMailReader *reader,
 
 	async_context = g_slice_new0 (AsyncContext);
 	async_context->activity = g_object_ref (activity);
-	async_context->folder = g_object_ref (folder);
+	async_context->folder = e_mail_reader_ref_folder (reader);
 	async_context->reader = g_object_ref (reader);
 	async_context->message_uid = g_strdup (message_list->cursor_uid);
 	async_context->print_action = action;
@@ -1101,14 +1117,9 @@ e_mail_reader_remove_attachments (EMailReader *reader)
 
 	g_return_if_fail (E_IS_MAIL_READER (reader));
 
-	folder = e_mail_reader_get_folder (reader);
 	uids = e_mail_reader_get_selected_uids (reader);
 	g_return_if_fail (uids != NULL);
 
-	/* XXX Either e_mail_reader_get_selected_uids()
-	 *     or MessageList should do this itself. */
-	g_ptr_array_set_free_func (uids, (GDestroyNotify) g_free);
-
 	/* Remove attachments asynchronously. */
 
 	activity = e_mail_reader_new_activity (reader);
@@ -1118,6 +1129,8 @@ e_mail_reader_remove_attachments (EMailReader *reader)
 	async_context->activity = g_object_ref (activity);
 	async_context->reader = g_object_ref (reader);
 
+	folder = e_mail_reader_ref_folder (reader);
+
 	e_mail_folder_remove_attachments (
 		folder, uids,
 		G_PRIORITY_DEFAULT,
@@ -1125,6 +1138,8 @@ e_mail_reader_remove_attachments (EMailReader *reader)
 		mail_reader_remove_attachments_cb,
 		async_context);
 
+	g_object_unref (folder);
+
 	g_object_unref (activity);
 
 	g_ptr_array_unref (uids);
@@ -1241,14 +1256,9 @@ e_mail_reader_remove_duplicates (EMailReader *reader)
 
 	g_return_if_fail (E_IS_MAIL_READER (reader));
 
-	folder = e_mail_reader_get_folder (reader);
 	uids = e_mail_reader_get_selected_uids (reader);
 	g_return_if_fail (uids != NULL);
 
-	/* XXX Either e_mail_reader_get_selected_uids()
-	 *     or MessageList should do this itself. */
-	g_ptr_array_set_free_func (uids, (GDestroyNotify) g_free);
-
 	/* Find duplicate messages asynchronously. */
 
 	activity = e_mail_reader_new_activity (reader);
@@ -1258,6 +1268,8 @@ e_mail_reader_remove_duplicates (EMailReader *reader)
 	async_context->activity = g_object_ref (activity);
 	async_context->reader = g_object_ref (reader);
 
+	folder = e_mail_reader_ref_folder (reader);
+
 	e_mail_folder_find_duplicate_messages (
 		folder, uids,
 		G_PRIORITY_DEFAULT,
@@ -1265,11 +1277,327 @@ e_mail_reader_remove_duplicates (EMailReader *reader)
 		mail_reader_remove_duplicates_cb,
 		async_context);
 
+	g_object_unref (folder);
+
 	g_object_unref (activity);
 
 	g_ptr_array_unref (uids);
 }
 
+static void
+mail_reader_edit_messages_cb (GObject *source_object,
+                              GAsyncResult *result,
+                              gpointer user_data)
+{
+	CamelFolder *folder;
+	EShell *shell;
+	EMailBackend *backend;
+	EActivity *activity;
+	EAlertSink *alert_sink;
+	GHashTable *hash_table;
+	GHashTableIter iter;
+	gpointer key, value;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	folder = CAMEL_FOLDER (source_object);
+	async_context = (AsyncContext *) user_data;
+
+	activity = async_context->activity;
+	alert_sink = e_activity_get_alert_sink (activity);
+
+	hash_table = e_mail_folder_get_multiple_messages_finish (
+		folder, result, &local_error);
+
+	/* Sanity check. */
+	g_return_if_fail (
+		((hash_table != NULL) && (local_error == NULL)) ||
+		((hash_table == NULL) && (local_error != NULL)));
+
+	if (e_activity_handle_cancellation (activity, local_error)) {
+		g_error_free (local_error);
+		goto exit;
+
+	} else if (local_error != NULL) {
+		e_alert_submit (
+			alert_sink,
+			"mail:get-multiple-messages",
+			local_error->message, NULL);
+		g_error_free (local_error);
+		goto exit;
+	}
+
+	backend = e_mail_reader_get_backend (async_context->reader);
+	shell = e_shell_backend_get_shell (E_SHELL_BACKEND (backend));
+
+	/* Open each message in its own composer window. */
+
+	g_hash_table_iter_init (&iter, hash_table);
+
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		EMsgComposer *composer;
+		CamelMimeMessage *message;
+		const gchar *message_uid = NULL;
+
+		if (async_context->replace)
+			message_uid = (const gchar *) key;
+
+		message = CAMEL_MIME_MESSAGE (value);
+
+		camel_medium_remove_header (
+			CAMEL_MEDIUM (message), "X-Mailer");
+
+		composer = em_utils_edit_message (
+			shell, folder, message, message_uid,
+			async_context->keep_signature);
+
+		e_mail_reader_composer_created (
+			async_context->reader, composer, message);
+	}
+
+	g_hash_table_unref (hash_table);
+
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
+
+exit:
+	async_context_free (async_context);
+}
+
+void
+e_mail_reader_edit_messages (EMailReader *reader,
+                             CamelFolder *folder,
+                             GPtrArray *uids,
+                             gboolean replace,
+                             gboolean keep_signature)
+{
+	EActivity *activity;
+	GCancellable *cancellable;
+	AsyncContext *async_context;
+
+	g_return_if_fail (E_IS_MAIL_READER (reader));
+	g_return_if_fail (CAMEL_IS_FOLDER (folder));
+	g_return_if_fail (uids != NULL);
+
+	activity = e_mail_reader_new_activity (reader);
+	cancellable = e_activity_get_cancellable (activity);
+
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->activity = g_object_ref (activity);
+	async_context->reader = g_object_ref (reader);
+	async_context->replace = replace;
+	async_context->keep_signature = keep_signature;
+
+	e_mail_folder_get_multiple_messages (
+		folder, uids,
+		G_PRIORITY_DEFAULT,
+		cancellable,
+		mail_reader_edit_messages_cb,
+		async_context);
+
+	g_object_unref (activity);
+}
+
+static void
+mail_reader_forward_attachment_cb (GObject *source_object,
+                                   GAsyncResult *result,
+                                   gpointer user_data)
+{
+	CamelFolder *folder;
+	EMailBackend *backend;
+	EActivity *activity;
+	EAlertSink *alert_sink;
+	CamelMimePart *part;
+	CamelDataWrapper *content;
+	EMsgComposer *composer;
+	gchar *subject = NULL;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	folder = CAMEL_FOLDER (source_object);
+	async_context = (AsyncContext *) user_data;
+
+	activity = async_context->activity;
+	alert_sink = e_activity_get_alert_sink (activity);
+
+	part = e_mail_folder_build_attachment_finish (
+		folder, result, &subject, &local_error);
+
+	/* Sanity check. */
+	g_return_if_fail (
+		((part != NULL) && (local_error == NULL)) ||
+		((part == NULL) && (local_error != NULL)));
+
+	if (e_activity_handle_cancellation (activity, local_error)) {
+		g_warn_if_fail (subject == NULL);
+		g_error_free (local_error);
+		goto exit;
+
+	} else if (local_error != NULL) {
+		g_warn_if_fail (subject == NULL);
+		e_alert_submit (
+			alert_sink,
+			"mail:get-multiple-messages",
+			local_error->message, NULL);
+		g_error_free (local_error);
+		goto exit;
+	}
+
+	backend = e_mail_reader_get_backend (async_context->reader);
+
+	composer = em_utils_forward_attachment (
+		backend, part, subject, folder, async_context->uids);
+
+	content = camel_medium_get_content (CAMEL_MEDIUM (part));
+	if (CAMEL_IS_MIME_MESSAGE (content)) {
+		e_mail_reader_composer_created (
+			async_context->reader, composer,
+			CAMEL_MIME_MESSAGE (content));
+	} else {
+		/* XXX What to do for the multipart/digest case?
+		 *     Extract the first message from the digest, or
+		 *     change the argument type to CamelMimePart and
+		 *     just pass the whole digest through?
+		 *
+		 *     This signal is primarily serving EMailBrowser,
+		 *     which can only forward one message at a time.
+		 *     So for the moment it doesn't matter, but still
+		 *     something to consider. */
+		e_mail_reader_composer_created (
+			async_context->reader, composer, NULL);
+	}
+
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
+
+	g_object_unref (part);
+	g_free (subject);
+
+exit:
+	async_context_free (async_context);
+}
+
+static void
+mail_reader_forward_messages_cb (GObject *source_object,
+                                 GAsyncResult *result,
+                                 gpointer user_data)
+{
+	CamelFolder *folder;
+	EMailBackend *backend;
+	EActivity *activity;
+	EAlertSink *alert_sink;
+	GHashTable *hash_table;
+	GHashTableIter iter;
+	gpointer key, value;
+	AsyncContext *async_context;
+	GError *local_error = NULL;
+
+	folder = CAMEL_FOLDER (source_object);
+	async_context = (AsyncContext *) user_data;
+
+	activity = async_context->activity;
+	alert_sink = e_activity_get_alert_sink (activity);
+
+	backend = e_mail_reader_get_backend (async_context->reader);
+
+	hash_table = e_mail_folder_get_multiple_messages_finish (
+		folder, result, &local_error);
+
+	/* Sanity check. */
+	g_return_if_fail (
+		((hash_table != NULL) && (local_error == NULL)) ||
+		((hash_table == NULL) && (local_error != NULL)));
+
+	if (e_activity_handle_cancellation (activity, local_error)) {
+		g_error_free (local_error);
+		goto exit;
+
+	} else if (local_error != NULL) {
+		e_alert_submit (
+			alert_sink,
+			"mail:get-multiple-messages",
+			local_error->message, NULL);
+		g_error_free (local_error);
+		goto exit;
+	}
+
+	/* Create a new composer window for each message. */
+
+	g_hash_table_iter_init (&iter, hash_table);
+
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		EMsgComposer *composer;
+		CamelMimeMessage *message;
+		const gchar *message_uid;
+
+		message_uid = (const gchar *) key;
+		message = CAMEL_MIME_MESSAGE (value);
+
+		composer = em_utils_forward_message (
+			backend, message,
+			async_context->forward_style,
+			folder, message_uid);
+
+		e_mail_reader_composer_created (
+			async_context->reader, composer, message);
+	}
+
+	g_hash_table_unref (hash_table);
+
+	e_activity_set_state (activity, E_ACTIVITY_COMPLETED);
+
+exit:
+	async_context_free (async_context);
+}
+
+void
+e_mail_reader_forward_messages (EMailReader *reader,
+                                CamelFolder *folder,
+                                GPtrArray *uids,
+                                EMailForwardStyle style)
+{
+	EActivity *activity;
+	GCancellable *cancellable;
+	AsyncContext *async_context;
+
+	g_return_if_fail (E_IS_MAIL_READER (reader));
+	g_return_if_fail (CAMEL_IS_FOLDER (folder));
+	g_return_if_fail (uids != NULL);
+
+	activity = e_mail_reader_new_activity (reader);
+	cancellable = e_activity_get_cancellable (activity);
+
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->activity = g_object_ref (activity);
+	async_context->reader = g_object_ref (reader);
+	async_context->uids = g_ptr_array_ref (uids);
+	async_context->forward_style = style;
+
+	switch (style) {
+		case E_MAIL_FORWARD_STYLE_ATTACHED:
+			e_mail_folder_build_attachment (
+				folder, uids,
+				G_PRIORITY_DEFAULT,
+				cancellable,
+				mail_reader_forward_attachment_cb,
+				async_context);
+			break;
+
+		case E_MAIL_FORWARD_STYLE_INLINE:
+		case E_MAIL_FORWARD_STYLE_QUOTED:
+			e_mail_folder_get_multiple_messages (
+				folder, uids,
+				G_PRIORITY_DEFAULT,
+				cancellable,
+				mail_reader_forward_messages_cb,
+				async_context);
+			break;
+
+		default:
+			g_warn_if_reached ();
+	}
+
+	g_object_unref (activity);
+}
+
 /* Helper for e_mail_reader_reply_to_message()
  * XXX This function belongs in e-html-utils.c */
 static gboolean
@@ -1322,6 +1650,7 @@ mail_reader_reply_message_parsed (GObject *object,
 	EMailBackend *backend;
 	EMailReader *reader = E_MAIL_READER (object);
 	EMailPartList *part_list;
+	EMsgComposer *composer;
 	CamelMimeMessage *message;
 	AsyncContext *async_context;
 
@@ -1333,7 +1662,7 @@ mail_reader_reply_message_parsed (GObject *object,
 	backend = e_mail_reader_get_backend (async_context->reader);
 	shell = e_shell_backend_get_shell (E_SHELL_BACKEND (backend));
 
-	em_utils_reply_to_message (
+	composer = em_utils_reply_to_message (
 		shell, message,
 		async_context->folder,
 		async_context->message_uid,
@@ -1342,6 +1671,8 @@ mail_reader_reply_message_parsed (GObject *object,
 		part_list,
 		async_context->address);
 
+	e_mail_reader_composer_created (reader, composer, message);
+
 	g_object_unref (part_list);
 
 	async_context_free (async_context);
@@ -1432,7 +1763,6 @@ e_mail_reader_reply_to_message (EMailReader *reader,
 	g_return_if_fail (E_IS_MAIL_READER (reader));
 
 	backend = e_mail_reader_get_backend (reader);
-	folder = e_mail_reader_get_folder (reader);
 	display = e_mail_reader_get_mail_display (reader);
 	message_list = e_mail_reader_get_message_list (reader);
 	reply_style = e_mail_reader_get_reply_style (reader);
@@ -1470,6 +1800,8 @@ e_mail_reader_reply_to_message (EMailReader *reader,
 	uid = MESSAGE_LIST (message_list)->cursor_uid;
 	g_return_if_fail (uid != NULL);
 
+	folder = e_mail_reader_ref_folder (reader);
+
 	if (!gtk_widget_get_visible (GTK_WIDGET (web_view)))
 		goto whole_message;
 
@@ -1558,7 +1890,7 @@ e_mail_reader_reply_to_message (EMailReader *reader,
 	composer = em_utils_reply_to_message (
 		shell, new_message, folder, uid,
 		reply_type, reply_style, NULL, address);
-	if (composer && (validity_pgp_sum != 0 || validity_smime_sum != 0)) {
+	if (validity_pgp_sum != 0 || validity_smime_sum != 0) {
 		GtkToggleAction *action;
 
 		if ((validity_pgp_sum & E_MAIL_PART_VALIDITY_PGP) != 0) {
@@ -1586,14 +1918,13 @@ e_mail_reader_reply_to_message (EMailReader *reader,
 		}
 	}
 
-	if (address)
-		g_object_unref (address);
+	e_mail_reader_composer_created (reader, composer, new_message);
 
 	g_object_unref (new_message);
 
 	g_free (selection);
 
-	return;
+	goto exit;
 
 whole_message:
 	if (src_message == NULL) {
@@ -1626,12 +1957,16 @@ whole_message:
 		g_object_unref (activity);
 
 	} else {
-		em_utils_reply_to_message (
+		composer = em_utils_reply_to_message (
 			shell, src_message, folder, uid,
 			reply_type, reply_style, part_list, address);
+
+		e_mail_reader_composer_created (reader, composer, src_message);
 	}
 
+exit:
 	g_clear_object (&address);
+	g_clear_object (&folder);
 }
 
 static void
@@ -1683,7 +2018,7 @@ e_mail_reader_save_messages (EMailReader *reader)
 	const gchar *title;
 	gchar *suggestion = NULL;
 
-	folder = e_mail_reader_get_folder (reader);
+	folder = e_mail_reader_ref_folder (reader);
 	backend = e_mail_reader_get_backend (reader);
 
 	uids = e_mail_reader_get_selected_uids (reader);
@@ -1698,10 +2033,6 @@ e_mail_reader_save_messages (EMailReader *reader)
 
 	message_uid = g_ptr_array_index (uids, 0);
 
-	/* XXX Either e_mail_reader_get_selected_uids()
-	 *     or MessageList should do this itself. */
-	g_ptr_array_set_free_func (uids, (GDestroyNotify) g_free);
-
 	title = ngettext ("Save Message", "Save Messages", uids->len);
 
 	/* Suggest as a filename the subject of the first message. */
@@ -1734,10 +2065,8 @@ e_mail_reader_save_messages (EMailReader *reader)
 		shell, title, suggestion,
 		"*.mbox:application/mbox,message/rfc822", NULL, NULL);
 
-	if (destination == NULL) {
-		g_ptr_array_unref (uids);
-		return;
-	}
+	if (destination == NULL)
+		goto exit;
 
 	/* Save messages asynchronously. */
 
@@ -1759,6 +2088,9 @@ e_mail_reader_save_messages (EMailReader *reader)
 	g_object_unref (activity);
 
 	g_object_unref (destination);
+
+exit:
+	g_clear_object (&folder);
 	g_ptr_array_unref (uids);
 }
 
@@ -1870,8 +2202,8 @@ e_mail_reader_create_filter_from_selected (EMailReader *reader,
 	shell = e_shell_backend_get_shell (E_SHELL_BACKEND (backend));
 	registry = e_shell_get_registry (shell);
 
-	folder = e_mail_reader_get_folder (reader);
-	g_return_if_fail (CAMEL_IS_FOLDER (folder));
+	folder = e_mail_reader_ref_folder (reader);
+	g_return_if_fail (folder != NULL);
 
 	if (em_utils_folder_is_sent (registry, folder) ||
 	    em_utils_folder_is_outbox (registry, folder))
@@ -1901,7 +2233,9 @@ e_mail_reader_create_filter_from_selected (EMailReader *reader,
 
 	g_object_unref (activity);
 
-	em_utils_uids_free (uids);
+	g_ptr_array_unref (uids);
+
+	g_object_unref (folder);
 }
 
 /* Helper for e_mail_reader_create_vfolder_from_selected() */
@@ -1987,15 +2321,11 @@ e_mail_reader_create_vfolder_from_selected (EMailReader *reader,
 	EActivity *activity;
 	GCancellable *cancellable;
 	AsyncContext *async_context;
-	CamelFolder *folder;
 	GPtrArray *uids;
 	const gchar *message_uid;
 
 	g_return_if_fail (E_IS_MAIL_READER (reader));
 
-	folder = e_mail_reader_get_folder (reader);
-	g_return_if_fail (CAMEL_IS_FOLDER (folder));
-
 	uids = e_mail_reader_get_selected_uids (reader);
 	g_return_if_fail (uids != NULL && uids->len == 1);
 	message_uid = g_ptr_array_index (uids, 0);
@@ -2005,13 +2335,14 @@ e_mail_reader_create_vfolder_from_selected (EMailReader *reader,
 
 	async_context = g_slice_new0 (AsyncContext);
 	async_context->activity = g_object_ref (activity);
-	async_context->folder = g_object_ref (folder);
+	async_context->folder = e_mail_reader_ref_folder (reader);
 	async_context->reader = g_object_ref (reader);
 	async_context->message_uid = g_strdup (message_uid);
 	async_context->filter_type = vfolder_type;
 
 	camel_folder_get_message (
-		folder, message_uid,
+		async_context->folder,
+		async_context->message_uid,
 		G_PRIORITY_DEFAULT,
 		cancellable,
 		mail_reader_create_vfolder_cb,
@@ -2019,116 +2350,7 @@ e_mail_reader_create_vfolder_from_selected (EMailReader *reader,
 
 	g_object_unref (activity);
 
-	em_utils_uids_free (uids);
-}
-
-static EMailReaderHeader *
-emr_header_from_xmldoc (xmlDocPtr doc)
-{
-	EMailReaderHeader *h;
-	xmlNodePtr root;
-	xmlChar *name;
-
-	if (doc == NULL)
-		return NULL;
-
-	root = doc->children;
-	if (strcmp ((gchar *) root->name, "header") != 0)
-		return NULL;
-
-	name = xmlGetProp (root, (const guchar *)"name");
-	if (name == NULL)
-		return NULL;
-
-	h = g_malloc0 (sizeof (EMailReaderHeader));
-	h->name = g_strdup ((gchar *) name);
-	xmlFree (name);
-
-	if (xmlHasProp (root, (const guchar *)"enabled"))
-		h->enabled = 1;
-	else
-		h->enabled = 0;
-
-	return h;
-}
-
-/**
- * e_mail_reader_header_from_xml
- * @xml: XML configuration data
- *
- * Parses passed XML data, which should be of
- * the format <header name="foo" enabled />, and
- * returns a EMailReaderHeader structure, or NULL if there
- * is an error.
- **/
-EMailReaderHeader *
-e_mail_reader_header_from_xml (const gchar *xml)
-{
-	EMailReaderHeader *header;
-	xmlDocPtr doc;
-
-	if (!(doc = xmlParseDoc ((guchar *) xml)))
-		return NULL;
-
-	header = emr_header_from_xmldoc (doc);
-	xmlFreeDoc (doc);
-
-	return header;
-}
-
-/**
- * e_mail_reader_header_to_xml
- * @header: header from which to generate XML
- *
- * Returns the passed header as a XML structure,
- * or NULL on error
- */
-gchar *
-e_mail_reader_header_to_xml (EMailReaderHeader *header)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	xmlChar *xml;
-	gchar *out;
-	gint size;
-
-	g_return_val_if_fail (header != NULL, NULL);
-	g_return_val_if_fail (header->name != NULL, NULL);
-
-	doc = xmlNewDoc ((const guchar *)"1.0");
-
-	root = xmlNewDocNode (doc, NULL, (const guchar *)"header", NULL);
-	xmlSetProp (root, (const guchar *)"name", (guchar *) header->name);
-	if (header->enabled)
-		xmlSetProp (root, (const guchar *)"enabled", NULL);
-
-	xmlDocSetRootElement (doc, root);
-	xmlDocDumpMemory (doc, &xml, &size);
-	xmlFreeDoc (doc);
-
-	out = g_malloc (size + 1);
-	memcpy (out, xml, size);
-	out[size] = '\0';
-	xmlFree (xml);
-
-	return out;
-}
-
-/**
- * e_mail_reader_header_free
- * @header: header to free
- *
- * Frees the memory associated with the passed header
- * structure.
- */
-void
-e_mail_reader_header_free (EMailReaderHeader *header)
-{
-	if (header == NULL)
-		return;
-
-	g_free (header->name);
-	g_free (header);
+	g_ptr_array_unref (uids);
 }
 
 static void
