@@ -142,10 +142,12 @@ handle_mail_request (GSimpleAsyncResult *res,
 		if (part != NULL) {
 			if (context.mode == E_MAIL_FORMATTER_MODE_CID) {
 				CamelDataWrapper *dw;
+				CamelMimePart *mime_part;
 				CamelStream *raw_content;
 				GByteArray *ba;
 
-				dw = camel_medium_get_content (CAMEL_MEDIUM (part->part));
+				mime_part = e_mail_part_ref_mime_part (part);
+				dw = camel_medium_get_content (CAMEL_MEDIUM (mime_part));
 				g_return_if_fail (dw);
 
 				raw_content = camel_stream_mem_new ();
@@ -155,13 +157,19 @@ handle_mail_request (GSimpleAsyncResult *res,
 				camel_stream_write (request->priv->output_stream, (gchar *) ba->data, ba->len, cancellable, NULL);
 
 				g_object_unref (raw_content);
+
+				g_object_unref (mime_part);
 			} else {
+				if (mime_type == NULL)
+					mime_type = e_mail_part_get_mime_type (part);
+
 				e_mail_formatter_format_as (
-					formatter, &context, part, request->priv->output_stream,
-					mime_type ? mime_type : part->mime_type, cancellable);
+					formatter, &context, part,
+					request->priv->output_stream,
+					mime_type, cancellable);
 			}
 
-			e_mail_part_unref (part);
+			g_object_unref (part);
 		} else {
 			g_warning ("Failed to lookup requested part '%s' - this should not happen!", part_id);
 		}
@@ -176,15 +184,9 @@ handle_mail_request (GSimpleAsyncResult *res,
 	context.part_list = NULL;
 
 	/* Convert the GString to GInputStream and send it back to WebKit */
-	ba = camel_stream_mem_get_byte_array (
-		CAMEL_STREAM_MEM (request->priv->output_stream));
-	if (ba->data == NULL) {
-		gchar *data;
-
-		/* XXX Text was added after 3.8.0; not translated. */
-		data = g_strdup_printf (
-			"<p align='center'>%s</p>",
-			"The message has no text content.");
+	ba = camel_stream_mem_get_byte_array (CAMEL_STREAM_MEM (request->priv->output_stream));
+	if (!ba->data) {
+		gchar *data = g_strdup_printf (_("Failed to load part '%s'"), part_id);
 		dd (printf ("%s", data));
 		g_byte_array_append (ba, (guchar *) data, strlen (data));
 		g_free (data);
