@@ -62,10 +62,6 @@
 #include "em-utils.h"
 #include "e-mail-printer.h"
 
-/* XXX This is a dirty hack on a dirty hack.  We really need
- *     to rework or get rid of the functions that use this. */
-extern const gchar *shell_builtin_backend;
-
 /* How many is too many? */
 /* Used in em_util_ask_open_many() */
 #define TOO_MANY 10
@@ -163,48 +159,6 @@ em_utils_prompt_user (GtkWindow *parent,
 	return button == GTK_RESPONSE_YES;
 }
 
-/**
- * em_utils_uids_copy:
- * @uids: array of uids
- *
- * Duplicates the array of uids held by @uids into a new
- * GPtrArray. Use em_utils_uids_free() to free the resultant uid
- * array.
- *
- * Returns a duplicate copy of @uids.
- **/
-GPtrArray *
-em_utils_uids_copy (GPtrArray *uids)
-{
-	GPtrArray *copy;
-	gint i;
-
-	copy = g_ptr_array_new ();
-	g_ptr_array_set_size (copy, uids->len);
-
-	for (i = 0; i < uids->len; i++)
-		copy->pdata[i] = g_strdup (uids->pdata[i]);
-
-	return copy;
-}
-
-/**
- * em_utils_uids_free:
- * @uids: array of uids
- *
- * Frees the array of uids pointed to by @uids back to the system.
- **/
-void
-em_utils_uids_free (GPtrArray *uids)
-{
-	gint i;
-
-	for (i = 0; i < uids->len; i++)
-		g_free (uids->pdata[i]);
-
-	g_ptr_array_free (uids, TRUE);
-}
-
 /* Editing Filters/Search Folders... */
 
 static GtkWidget *filter_editor = NULL;
@@ -435,9 +389,6 @@ em_utils_flag_for_followup (EMailReader *reader,
 	e_mail_display_reload (display);
 
 exit:
-	/* XXX We shouldn't be freeing this. */
-	em_utils_uids_free (uids);
-
 	gtk_widget_destroy (GTK_WIDGET (editor));
 }
 
@@ -474,8 +425,6 @@ em_utils_flag_for_followup_clear (GtkWindow *parent,
 	}
 
 	camel_folder_thaw (folder);
-
-	em_utils_uids_free (uids);
 }
 
 /**
@@ -518,8 +467,6 @@ em_utils_flag_for_followup_completed (GtkWindow *parent,
 	camel_folder_thaw (folder);
 
 	g_free (now);
-
-	em_utils_uids_free (uids);
 }
 
 /* This kind of sucks, because for various reasons most callers need to run
@@ -880,6 +827,7 @@ em_utils_selection_get_uidlist (GtkSelectionData *selection_data,
 		return;
 
 	items = g_ptr_array_new ();
+	g_ptr_array_set_free_func (items, (GDestroyNotify) g_free);
 
 	inptr = (gchar *) data;
 	inend = (gchar *) (data + length);
@@ -895,7 +843,7 @@ em_utils_selection_get_uidlist (GtkSelectionData *selection_data,
 	}
 
 	if (items->len == 0) {
-		g_ptr_array_free (items, TRUE);
+		g_ptr_array_unref (items);
 		return;
 	}
 
@@ -938,7 +886,7 @@ em_utils_selection_get_uidlist (GtkSelectionData *selection_data,
 	}
 
 	g_hash_table_destroy (uids_by_uri);
-	em_utils_uids_free (items);
+	g_ptr_array_unref (items);
 
 	if (local_error)
 		g_propagate_error (error, local_error);
@@ -1519,8 +1467,7 @@ check_prefix (const gchar *subject,
 }
 
 gboolean
-em_utils_is_re_in_subject (EShell *shell,
-                           const gchar *subject,
+em_utils_is_re_in_subject (const gchar *subject,
                            gint *skip_len)
 {
 	GSettings *settings;
@@ -1528,7 +1475,6 @@ em_utils_is_re_in_subject (EShell *shell,
 	gboolean res;
 	gint ii;
 
-	g_return_val_if_fail (shell != NULL, FALSE);
 	g_return_val_if_fail (subject != NULL, FALSE);
 	g_return_val_if_fail (skip_len != NULL, FALSE);
 
