@@ -100,13 +100,8 @@ struct _MessageList {
 	MessageListPrivate *priv;
 
 	/* The table */
-	ETreeModel   *model;
-	ETreePath     tree_root;
 	ETableExtras *extras;
 
-	/* The folder being shown. */
-	CamelFolder *folder;
-
 	GHashTable *uid_nodemap; /* uid (from info) -> tree node mapping */
 
 	GHashTable *normalised_hash;
@@ -114,26 +109,13 @@ struct _MessageList {
 	/* Current search string, or %NULL */
 	gchar *search;
 
-	/* which message uid should be left in the list even not in a
-	 * search after rebuild; rebuild will clear the value to NULL */
-	gchar *ensure_uid;
-
 	/* are we regenerating the message_list because set_folder
 	 * was just called? */
 	guint just_set_folder : 1;
 
-	/* Are we displaying threaded view? */
-	guint threaded : 1;
-
 	guint expand_all :1;
 	guint collapse_all :1;
 
-	/* do we automatically hide deleted messages? */
-	guint hidedeleted : 1;
-
-	/* do we automatically hide junk messages? */
-	guint hidejunk : 1;
-
 	/* frozen count */
 	guint frozen : 16;
 
@@ -146,15 +128,6 @@ struct _MessageList {
 	/* Row-selection and seen-marking timers */
 	guint idle_id, seen_id;
 
-	/* list of outstanding regeneration requests */
-	GList *regen;
-	GMutex regen_lock; /* when writing to the regen, guard with this lock too */
-	gchar *pending_select_uid;	/* set if we were busy regnerating
-					 * while we had a select come in */
-	gboolean pending_select_fallback;
-	guint regen_timeout_id;
-	gpointer regen_timeout_msg;
-
 	gchar *frozen_search;	/* to save search took place while we were frozen */
 
 	/* the current camel folder thread tree, if any */
@@ -180,16 +153,32 @@ typedef enum {
 GType		message_list_get_type		(void);
 GtkWidget *	message_list_new		(EMailSession *session);
 EMailSession *	message_list_get_session	(MessageList *message_list);
+CamelFolder *	message_list_ref_folder		(MessageList *message_list);
 void		message_list_set_folder		(MessageList *message_list,
-						 CamelFolder *folder,
-						 gboolean outgoing);
+						 CamelFolder *folder);
 GtkTargetList *	message_list_get_copy_target_list
 						(MessageList *message_list);
 GtkTargetList *	message_list_get_paste_target_list
 						(MessageList *message_list);
+void		message_list_set_expanded_default
+						(MessageList *message_list,
+						 gboolean expanded_default);
+gboolean	message_list_get_group_by_threads
+						(MessageList *message_list);
+void		message_list_set_group_by_threads
+						(MessageList *message_list,
+						 gboolean group_by_threads);
+gboolean	message_list_get_show_deleted	(MessageList *message_list);
+void		message_list_set_show_deleted	(MessageList *message_list,
+						 gboolean show_deleted);
+gboolean	message_list_get_thread_latest	(MessageList *message_list);
+void		message_list_set_thread_latest	(MessageList *message_list,
+						 gboolean thread_latest);
+gboolean	message_list_get_thread_subject	(MessageList *message_list);
+void		message_list_set_thread_subject	(MessageList *message_list,
+						 gboolean thread_subject);
 void		message_list_freeze		(MessageList *message_list);
 void		message_list_thaw		(MessageList *message_list);
-GPtrArray *	message_list_get_uids		(MessageList *message_list);
 GPtrArray *	message_list_get_selected	(MessageList *message_list);
 void		message_list_set_selected	(MessageList *message_list,
 						 GPtrArray *uids);
@@ -215,22 +204,16 @@ void		message_list_copy		(MessageList *message_list,
 void		message_list_paste		(MessageList *message_list);
 guint		message_list_count		(MessageList *message_list);
 guint		message_list_selected_count	(MessageList *message_list);
-void		message_list_set_threaded	(MessageList *message_list,
-						 gboolean threaded);
 void		message_list_set_threaded_expand_all
 						(MessageList *message_list);
 void		message_list_set_threaded_collapse_all
 						(MessageList *message_list);
-void		message_list_set_hidedeleted	(MessageList *message_list,
-						 gboolean hidedeleted);
 void		message_list_set_search		(MessageList *message_list,
 						 const gchar *search);
-void		message_list_ensure_message	(MessageList *message_list,
-						 const gchar *uid);
 void		message_list_save_state		(MessageList *message_list);
 
-#define MESSAGE_LIST_LOCK(m, l) g_mutex_lock(((MessageList *)m)->l)
-#define MESSAGE_LIST_UNLOCK(m, l) g_mutex_unlock(((MessageList *)m)->l)
+void		message_list_sort_uids		(MessageList *message_list,
+						 GPtrArray *uids);
 
 G_END_DECLS
 
