@@ -335,9 +335,8 @@ cal_shell_view_selector_client_added_cb (ECalShellView *cal_shell_view,
 	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
 	model = gnome_calendar_get_model (calendar);
 
-	e_cal_model_add_client (model, client);
-
-	gnome_calendar_update_query (calendar);
+	if (e_cal_model_add_client (model, client))
+		gnome_calendar_update_query (calendar);
 }
 
 static void
@@ -352,9 +351,8 @@ cal_shell_view_selector_client_removed_cb (ECalShellView *cal_shell_view,
 	calendar = e_cal_shell_content_get_calendar (cal_shell_content);
 	model = gnome_calendar_get_model (calendar);
 
-	e_cal_model_remove_client (model, client);
-
-	gnome_calendar_update_query (calendar);
+	if (e_cal_model_remove_client (model, client))
+		gnome_calendar_update_query (calendar);
 }
 
 static void
@@ -379,18 +377,14 @@ cal_shell_view_taskpad_popup_event_cb (EShellView *shell_view,
 
 static void
 cal_shell_view_user_created_cb (ECalShellView *cal_shell_view,
-                                ECalClient *where_was_created,
+                                ECalClient *client,
                                 ECalendarView *calendar_view)
 {
 	ECalShellSidebar *cal_shell_sidebar;
-	ESource *source;
-
-	g_return_if_fail (where_was_created != NULL);
-
-	source = e_client_get_source (E_CLIENT (where_was_created));
 
 	cal_shell_sidebar = cal_shell_view->priv->cal_shell_sidebar;
-	e_cal_shell_sidebar_add_source (cal_shell_sidebar, source);
+
+	e_cal_shell_sidebar_add_client (cal_shell_sidebar, E_CLIENT (client));
 }
 
 static void
@@ -586,10 +580,8 @@ e_cal_shell_view_private_constructed (ECalShellView *cal_shell_view)
 
 	/* Give GnomeCalendar a handle to the date navigator, memo and task table. */
 	gnome_calendar_set_date_navigator (calendar, date_navigator);
-	gnome_calendar_set_memo_table (
-		calendar, memo_table ? GTK_WIDGET (memo_table) : NULL);
-	gnome_calendar_set_task_table (
-		calendar, task_table ? GTK_WIDGET (task_table) : NULL);
+	gnome_calendar_set_memo_table (calendar, GTK_WIDGET (memo_table));
+	gnome_calendar_set_task_table (calendar, GTK_WIDGET (task_table));
 
 	e_calendar_item_set_get_time_callback (
 		date_navigator->calitem, (ECalendarItemGetTimeCallback)
@@ -742,6 +734,13 @@ e_cal_shell_view_private_dispose (ECalShellView *cal_shell_view)
 	if (priv->cal_shell_content != NULL)
 		e_cal_shell_content_save_state (priv->cal_shell_content);
 
+	if (priv->prepare_for_quit_handler_id > 0) {
+		g_signal_handler_disconnect (
+			priv->shell,
+			priv->prepare_for_quit_handler_id);
+		priv->prepare_for_quit_handler_id = 0;
+	}
+
 	if (priv->backend_error_handler_id > 0) {
 		g_signal_handler_disconnect (
 			priv->client_cache,
@@ -753,6 +752,7 @@ e_cal_shell_view_private_dispose (ECalShellView *cal_shell_view)
 	g_clear_object (&priv->cal_shell_content);
 	g_clear_object (&priv->cal_shell_sidebar);
 
+	g_clear_object (&priv->shell);
 	g_clear_object (&priv->client_cache);
 
 	if (priv->calendar_activity != NULL) {
@@ -1458,6 +1458,8 @@ cal_iterate_searching (ECalShellView *cal_shell_view)
 			priv->searching_activity = NULL;
 		}
 
+		e_shell_view_update_actions (E_SHELL_VIEW (cal_shell_view));
+
 		return;
 	}
 
@@ -1564,7 +1566,7 @@ cal_iterate_searching (ECalShellView *cal_shell_view)
 	priv->search_time = new_time;
 
 	for (link = list; link != NULL; link = g_list_next (link)) {
-		ECalClient *client = E_CAL_CLIENT (link);
+		ECalClient *client = E_CAL_CLIENT (link->data);
 
 		e_cal_client_get_object_list (
 			client, sexp, cancellable,
@@ -1573,6 +1575,8 @@ cal_iterate_searching (ECalShellView *cal_shell_view)
 
 	g_list_free_full (list, (GDestroyNotify) g_object_unref);
 	g_free (sexp);
+
+	e_shell_view_update_actions (E_SHELL_VIEW (cal_shell_view));
 }
 
 void
@@ -1621,8 +1625,10 @@ e_cal_shell_view_search_events (ECalShellView *cal_shell_view,
 
 	priv->search_direction = search_forward ? +30 : -30;
 
-	if (cal_searching_check_candidates (cal_shell_view))
+	if (cal_searching_check_candidates (cal_shell_view)) {
+		e_shell_view_update_actions (E_SHELL_VIEW (cal_shell_view));
 		return;
+	}
 
 	range_years = cal_searching_get_search_range_years (cal_shell_view);
 
