@@ -435,6 +435,12 @@ task_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 		G_CALLBACK (task_shell_sidebar_primary_selection_changed_cb),
 		shell_sidebar);
 
+	/* This will trigger our "row-changed" signal handler for each
+	 * task list source, so the appropriate ECalClients get added to
+	 * the ECalModel, which will then create view objects to display
+	 * the task list content.  This all happens asynchronously. */
+	e_source_selector_update_all_rows (selector);
+
 	/* Bind GObject properties to settings keys. */
 
 	settings = g_settings_new ("org.gnome.evolution.calendar");
