@@ -317,32 +317,29 @@ static void
 action_task_list_refresh_cb (GtkAction *action,
                              ETaskShellView *task_shell_view)
 {
-	ETaskShellContent *task_shell_content;
 	ETaskShellSidebar *task_shell_sidebar;
 	ESourceSelector *selector;
-	ECalClient *client;
-	ECalModel *model;
+	EClient *client = NULL;
 	ESource *source;
 	GError *error = NULL;
 
-	task_shell_content = task_shell_view->priv->task_shell_content;
 	task_shell_sidebar = task_shell_view->priv->task_shell_sidebar;
-
-	model = e_task_shell_content_get_task_model (task_shell_content);
 	selector = e_task_shell_sidebar_get_selector (task_shell_sidebar);
 
 	source = e_source_selector_ref_primary_selection (selector);
-	g_return_if_fail (source != NULL);
 
-	client = e_cal_model_get_client_for_source (model, source);
-	if (client == NULL) {
+	if (source != NULL) {
+		client = e_client_selector_ref_cached_client (
+			E_CLIENT_SELECTOR (selector), source);
 		g_object_unref (source);
-		return;
 	}
 
-	g_return_if_fail (e_client_check_refresh_supported (E_CLIENT (client)));
+	if (client == NULL)
+		return;
 
-	e_client_refresh_sync (E_CLIENT (client), NULL, &error);
+	g_return_if_fail (e_client_check_refresh_supported (client));
+
+	e_client_refresh_sync (client, NULL, &error);
 
 	if (error != NULL) {
 		g_warning (
@@ -352,7 +349,7 @@ action_task_list_refresh_cb (GtkAction *action,
 		g_error_free (error);
 	}
 
-	g_object_unref (source);
+	g_object_unref (client);
 }
 
 static void
@@ -459,12 +456,12 @@ action_task_new_cb (GtkAction *action,
 		ECalModel *model;
 
 		model = e_task_table_get_model (task_table);
-		client = e_cal_model_get_default_client (model);
+		client = e_cal_model_ref_default_client (model);
 	} else {
 		ECalModelComponent *comp_data;
 
 		comp_data = list->data;
-		client = comp_data->client;
+		client = g_object_ref (comp_data->client);
 		g_slist_free (list);
 	}
 
@@ -477,6 +474,8 @@ action_task_new_cb (GtkAction *action,
 	gtk_window_present (GTK_WINDOW (editor));
 
 	g_object_unref (comp);
+
+	g_object_unref (client);
 }
 
 static void
