@@ -33,15 +33,13 @@ task_shell_view_model_row_appended_cb (ETaskShellView *task_shell_view,
 {
 	ETaskShellSidebar *task_shell_sidebar;
 	ECalClient *client;
-	ESource *source;
 
 	/* This is the "Click to Add" handler. */
 
 	client = e_cal_model_ref_default_client (model);
-	source = e_client_get_source (E_CLIENT (client));
 
 	task_shell_sidebar = task_shell_view->priv->task_shell_sidebar;
-	e_task_shell_sidebar_add_source (task_shell_sidebar, source);
+	e_task_shell_sidebar_add_client (task_shell_sidebar, E_CLIENT (client));
 
 	g_object_unref (client);
 }
