@@ -62,7 +62,8 @@ emfe_itip_format (EMailFormatterExtension *extension,
 	GString *buffer;
 	EMailPartItip *itip_part;
 
-	g_return_val_if_fail (E_MAIL_PART_IS (part, EMailPartItip), FALSE);
+	g_return_val_if_fail (E_IS_MAIL_PART_ITIP (part), FALSE);
+
 	itip_part = (EMailPartItip *) part;
 
 	if (context->mode == E_MAIL_FORMATTER_MODE_PRINTING) {
@@ -114,7 +115,7 @@ emfe_itip_format (EMailFormatterExtension *extension,
 
 		uri = e_mail_part_build_uri (
 			folder, message_uid,
-			"part_id", G_TYPE_STRING, part->id,
+			"part_id", G_TYPE_STRING, e_mail_part_get_id (part),
 			"mode", G_TYPE_INT, E_MAIL_FORMATTER_MODE_RAW,
 			"formatter_default_charset", G_TYPE_STRING, default_charset,
 			"formatter_charset", G_TYPE_STRING, charset,
@@ -128,7 +129,9 @@ emfe_itip_format (EMailFormatterExtension *extension,
 			"<iframe width=\"100%%\" height=\"auto\""
 			" frameborder=\"0\" src=\"%s\" name=\"%s\" id=\"%s\"></iframe>"
 			"</div>",
-			uri, part->id, part->id);
+			uri,
+			e_mail_part_get_id (part),
+			e_mail_part_get_id (part));
 
 		g_free (uri);
 	}
