@@ -227,10 +227,6 @@ void		e_shell_view_unblock_execute_search
 gboolean	e_shell_view_is_execute_search_blocked
 						(EShellView *shell_view);
 void		e_shell_view_update_actions	(EShellView *shell_view);
-void		e_shell_view_block_update_actions
-						(EShellView *shell_view);
-void		e_shell_view_unblock_update_actions
-						(EShellView *shell_view);
 GtkWidget *	e_shell_view_show_popup_menu	(EShellView *shell_view,
 						 const gchar *widget_path,
 						 GdkEvent *button_event);
