@@ -220,7 +220,7 @@ e_shell_window_private_init (EShellWindow *shell_window)
 
 	signal_handler_ids = g_array_new (FALSE, FALSE, sizeof (gulong));
 
-	priv->ui_manager = e_ui_manager_new ();
+	priv->ui_manager = gtk_ui_manager_new ();
 	priv->loaded_views = loaded_views;
 	priv->signal_handler_ids = signal_handler_ids;
 
@@ -272,7 +272,6 @@ e_shell_window_private_constructed (EShellWindow *shell_window)
 	shell = e_shell_window_get_shell (shell_window);
 
 	ui_manager = e_shell_window_get_ui_manager (shell_window);
-	e_shell_configure_ui_manager (shell, E_UI_MANAGER (ui_manager));
 
 	/* Defer actions and menu merging until we have set express mode */
 
@@ -433,14 +432,10 @@ e_shell_window_private_constructed (EShellWindow *shell_window)
 		shell_window, "taskbar-visible",
 		G_SETTINGS_BIND_DEFAULT);
 
-	if (e_shell_get_express_mode (shell)) {
-		e_shell_window_set_switcher_visible (shell_window, FALSE);
-	} else {
-		g_settings_bind (
-			settings, "buttons-visible",
-			shell_window, "switcher-visible",
-			G_SETTINGS_BIND_DEFAULT);
-	}
+	g_settings_bind (
+		settings, "buttons-visible",
+		shell_window, "switcher-visible",
+		G_SETTINGS_BIND_DEFAULT);
 
 	g_settings_bind (
 		settings, "toolbar-visible",
